searchData={"items":[{"type":"module","title":"ErrorMessage","doc":"ErrorMessage\n===\n[![Coverage](https://github.com/MikaAK/elixir_error_message/actions/workflows/coverage.yml/badge.svg)](https://github.com/MikaAK/elixir_error_message/actions/workflows/coverage.yml)\n[![Test](https://github.com/MikaAK/elixir_error_message/actions/workflows/test.yml/badge.svg)](https://github.com/MikaAK/elixir_error_message/actions/workflows/test.yml)\n[![Dialyzer](https://github.com/MikaAK/elixir_error_message/actions/workflows/dialyzer.yml/badge.svg)](https://github.com/MikaAK/elixir_error_message/actions/workflows/dialyzer.yml)\n[![Credo](https://github.com/MikaAK/elixir_error_message/actions/workflows/credo.yml/badge.svg)](https://github.com/MikaAK/elixir_error_message/actions/workflows/credo.yml)\n[![codecov](https://codecov.io/gh/MikaAK/elixir_error_message/branch/master/graph/badge.svg?token=V0JJA5AZ1H)](https://codecov.io/gh/MikaAK/elixir_error_message)\n[![Hex pm](http://img.shields.io/hexpm/v/error_message.svg?style=flat)](https://hex.pm/packages/error_message)\n\nThis library exists to simplify error systems in a code base\nand allow for a simple unified experience when using and reading\nerror messages around the code base\n\nThis creates one standard, that all errors should fit into the context\nof HTTP error codes, if they don't `:internal_server_error` should\nbe used and you can use the message and details to provide a further\nlevel of depth","ref":"ErrorMessage.html"},{"type":"module","title":"Installation - ErrorMessage","doc":"The package can be installed by adding `error_message` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:error_message, \"~> 0.2.0\"}\n  ]\nend\n```\n\nDocumentation can be found at [https://hexdocs.pm/error_message](https://hexdocs.pm/error_message).","ref":"ErrorMessage.html#module-installation"},{"type":"module","title":"Usage Example - ErrorMessage","doc":"```elixir\niex> id = 1\niex> ErrorMessage.not_found(\"no user with id #{id}\", %{user_id: id})\n%ErrorMessage{\n  code: :not_found,\n  message: \"no user with id 1\",\n  details: %{user_id: 1}\n}\n\niex> ErrorMessage.internal_server_error(\"critical internal error\", %{\n...>   reason: :massive_issue_with_x\n...> })\n%ErrorMessage{\n  code: :internal_server_error,\n  message: \"critical internal error\",\n  details: %{reason: :massive_issue_with_x}\n}\n```","ref":"ErrorMessage.html#module-usage-example"},{"type":"module","title":"Why is this important - ErrorMessage","doc":"If we want to have a good way to catch errors around our system as well as be able to\ndisplay errors that happen throughout our system, it's useful to have a common error\napi so we can predict what will come out of a system\n\nFor example if we used elixir through our server, we would be able to catch a not found\npretty easily since we can predict the error code coming in without needing to\nknow the message. This leads to more resilliant code since message changes won't break\nthe functionality of your application\n\n```elixir\n# Because our error system is setup with `find_user` we can easily\n# catch no users and have a solid backup plan\nwith {:error, %ErrorMessage{code: :not_found}} <- find_user(%{name: \"bill\"}) do\n  create_user(%{name: \"bill\"})\nend\n```","ref":"ErrorMessage.html#module-why-is-this-important"},{"type":"module","title":"Usage with Phoenix - ErrorMessage","doc":"Another benefit is error rendering to the frontend, because all our errors are part of\nthe HTTP error system, it's quite easy to now return the proper status codes and messages\nto our frontend clients. For example:\n\n```elixir\ndefmodule MyController do\n  def index(conn, param) do\n    case find_thing(params) do\n      {:ok, res} -> json(conn, res)\n      {:error, e} -> json_error(conn, e)\n    end\n  end\n\n  defp json_error(conn, %ErrorMessage{code: code} = e) do\n    conn\n      |> put_status(code) # Plug.Conn\n      |> json(ErrorMessage.to_jsonable_map(e))\n  end\nend\n```\n\n*This will also add a request_id from `Logger.metadata[:request_id]` when found*","ref":"ErrorMessage.html#module-usage-with-phoenix"},{"type":"module","title":"Usage with Logger - ErrorMessage","doc":"Ontop of being useful for Phoenix we can also find some good use from this\nsystem and Logger, since `ErrorMessage` implements `String.Chars` protocol\n\n```elixir\ncase do_thing() do\n  {:ok, value} -> {:ok, do_other_thing(value)}\n  {:error, e} = res ->\n    Logger.error(\"[MyModule] \\#{e}\")\n    Logger.warn(to_string(e))\n\n    res\nend\n\n```","ref":"ErrorMessage.html#module-usage-with-logger"},{"type":"function","title":"ErrorMessage.bad_gateway/1","doc":"Create bad_gateway error message for status code 502","ref":"ErrorMessage.html#bad_gateway/1"},{"type":"function","title":"Example - ErrorMessage.bad_gateway/1","doc":"iex> ErrorMessage.bad_gateway(\"error message\")\n    %ErrorMessage{code: :bad_gateway, message: \"error message\"}","ref":"ErrorMessage.html#bad_gateway/1-example"},{"type":"function","title":"ErrorMessage.bad_gateway/2","doc":"Create bad_gateway error message for status code 502 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#bad_gateway/2"},{"type":"function","title":"Example - ErrorMessage.bad_gateway/2","doc":"iex> ErrorMessage.bad_gateway(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :bad_gateway, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#bad_gateway/2-example"},{"type":"function","title":"ErrorMessage.bad_request/1","doc":"Create bad_request error message for status code 400","ref":"ErrorMessage.html#bad_request/1"},{"type":"function","title":"Example - ErrorMessage.bad_request/1","doc":"iex> ErrorMessage.bad_request(\"error message\")\n    %ErrorMessage{code: :bad_request, message: \"error message\"}","ref":"ErrorMessage.html#bad_request/1-example"},{"type":"function","title":"ErrorMessage.bad_request/2","doc":"Create bad_request error message for status code 400 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#bad_request/2"},{"type":"function","title":"Example - ErrorMessage.bad_request/2","doc":"iex> ErrorMessage.bad_request(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :bad_request, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#bad_request/2-example"},{"type":"type","title":"ErrorMessage.code/0","doc":"","ref":"ErrorMessage.html#t:code/0"},{"type":"function","title":"ErrorMessage.conflict/1","doc":"Create conflict error message for status code 409","ref":"ErrorMessage.html#conflict/1"},{"type":"function","title":"Example - ErrorMessage.conflict/1","doc":"iex> ErrorMessage.conflict(\"error message\")\n    %ErrorMessage{code: :conflict, message: \"error message\"}","ref":"ErrorMessage.html#conflict/1-example"},{"type":"function","title":"ErrorMessage.conflict/2","doc":"Create conflict error message for status code 409 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#conflict/2"},{"type":"function","title":"Example - ErrorMessage.conflict/2","doc":"iex> ErrorMessage.conflict(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :conflict, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#conflict/2-example"},{"type":"function","title":"ErrorMessage.expectation_failed/1","doc":"Create expectation_failed error message for status code 417","ref":"ErrorMessage.html#expectation_failed/1"},{"type":"function","title":"Example - ErrorMessage.expectation_failed/1","doc":"iex> ErrorMessage.expectation_failed(\"error message\")\n    %ErrorMessage{code: :expectation_failed, message: \"error message\"}","ref":"ErrorMessage.html#expectation_failed/1-example"},{"type":"function","title":"ErrorMessage.expectation_failed/2","doc":"Create expectation_failed error message for status code 417 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#expectation_failed/2"},{"type":"function","title":"Example - ErrorMessage.expectation_failed/2","doc":"iex> ErrorMessage.expectation_failed(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :expectation_failed, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#expectation_failed/2-example"},{"type":"function","title":"ErrorMessage.failed_dependency/1","doc":"Create failed_dependency error message for status code 424","ref":"ErrorMessage.html#failed_dependency/1"},{"type":"function","title":"Example - ErrorMessage.failed_dependency/1","doc":"iex> ErrorMessage.failed_dependency(\"error message\")\n    %ErrorMessage{code: :failed_dependency, message: \"error message\"}","ref":"ErrorMessage.html#failed_dependency/1-example"},{"type":"function","title":"ErrorMessage.failed_dependency/2","doc":"Create failed_dependency error message for status code 424 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#failed_dependency/2"},{"type":"function","title":"Example - ErrorMessage.failed_dependency/2","doc":"iex> ErrorMessage.failed_dependency(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :failed_dependency, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#failed_dependency/2-example"},{"type":"function","title":"ErrorMessage.forbidden/1","doc":"Create forbidden error message for status code 403","ref":"ErrorMessage.html#forbidden/1"},{"type":"function","title":"Example - ErrorMessage.forbidden/1","doc":"iex> ErrorMessage.forbidden(\"error message\")\n    %ErrorMessage{code: :forbidden, message: \"error message\"}","ref":"ErrorMessage.html#forbidden/1-example"},{"type":"function","title":"ErrorMessage.forbidden/2","doc":"Create forbidden error message for status code 403 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#forbidden/2"},{"type":"function","title":"Example - ErrorMessage.forbidden/2","doc":"iex> ErrorMessage.forbidden(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :forbidden, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#forbidden/2-example"},{"type":"function","title":"ErrorMessage.found/1","doc":"Create found error message for status code 302","ref":"ErrorMessage.html#found/1"},{"type":"function","title":"Example - ErrorMessage.found/1","doc":"iex> ErrorMessage.found(\"error message\")\n    %ErrorMessage{code: :found, message: \"error message\"}","ref":"ErrorMessage.html#found/1-example"},{"type":"function","title":"ErrorMessage.found/2","doc":"Create found error message for status code 302 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#found/2"},{"type":"function","title":"Example - ErrorMessage.found/2","doc":"iex> ErrorMessage.found(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :found, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#found/2-example"},{"type":"function","title":"ErrorMessage.gateway_timeout/1","doc":"Create gateway_timeout error message for status code 504","ref":"ErrorMessage.html#gateway_timeout/1"},{"type":"function","title":"Example - ErrorMessage.gateway_timeout/1","doc":"iex> ErrorMessage.gateway_timeout(\"error message\")\n    %ErrorMessage{code: :gateway_timeout, message: \"error message\"}","ref":"ErrorMessage.html#gateway_timeout/1-example"},{"type":"function","title":"ErrorMessage.gateway_timeout/2","doc":"Create gateway_timeout error message for status code 504 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#gateway_timeout/2"},{"type":"function","title":"Example - ErrorMessage.gateway_timeout/2","doc":"iex> ErrorMessage.gateway_timeout(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :gateway_timeout, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#gateway_timeout/2-example"},{"type":"function","title":"ErrorMessage.gone/1","doc":"Create gone error message for status code 410","ref":"ErrorMessage.html#gone/1"},{"type":"function","title":"Example - ErrorMessage.gone/1","doc":"iex> ErrorMessage.gone(\"error message\")\n    %ErrorMessage{code: :gone, message: \"error message\"}","ref":"ErrorMessage.html#gone/1-example"},{"type":"function","title":"ErrorMessage.gone/2","doc":"Create gone error message for status code 410 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#gone/2"},{"type":"function","title":"Example - ErrorMessage.gone/2","doc":"iex> ErrorMessage.gone(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :gone, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#gone/2-example"},{"type":"function","title":"ErrorMessage.http_code/1","doc":"Returns the http code for an error message or error code atom","ref":"ErrorMessage.html#http_code/1"},{"type":"function","title":"Example - ErrorMessage.http_code/1","doc":"iex> ErrorMessage.http_code(:internal_server_error)\n    500\n\n    iex> ErrorMessage.http_code(ErrorMessage.not_found(\"some_message\"))\n    404","ref":"ErrorMessage.html#http_code/1-example"},{"type":"function","title":"ErrorMessage.http_code_reason_atom/1","doc":"Returns the http reason as an atom for the http error code","ref":"ErrorMessage.html#http_code_reason_atom/1"},{"type":"function","title":"Example - ErrorMessage.http_code_reason_atom/1","doc":"iex> ErrorMessage.http_code_reason_atom(500)\n    :internal_server_error","ref":"ErrorMessage.html#http_code_reason_atom/1-example"},{"type":"function","title":"ErrorMessage.http_version_not_supported/1","doc":"Create http_version_not_supported error message for status code 505","ref":"ErrorMessage.html#http_version_not_supported/1"},{"type":"function","title":"Example - ErrorMessage.http_version_not_supported/1","doc":"iex> ErrorMessage.http_version_not_supported(\"error message\")\n    %ErrorMessage{code: :http_version_not_supported, message: \"error message\"}","ref":"ErrorMessage.html#http_version_not_supported/1-example"},{"type":"function","title":"ErrorMessage.http_version_not_supported/2","doc":"Create http_version_not_supported error message for status code 505 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#http_version_not_supported/2"},{"type":"function","title":"Example - ErrorMessage.http_version_not_supported/2","doc":"iex> ErrorMessage.http_version_not_supported(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :http_version_not_supported, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#http_version_not_supported/2-example"},{"type":"function","title":"ErrorMessage.im_a_teapot/1","doc":"Create im_a_teapot error message for status code 418","ref":"ErrorMessage.html#im_a_teapot/1"},{"type":"function","title":"Example - ErrorMessage.im_a_teapot/1","doc":"iex> ErrorMessage.im_a_teapot(\"error message\")\n    %ErrorMessage{code: :im_a_teapot, message: \"error message\"}","ref":"ErrorMessage.html#im_a_teapot/1-example"},{"type":"function","title":"ErrorMessage.im_a_teapot/2","doc":"Create im_a_teapot error message for status code 418 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#im_a_teapot/2"},{"type":"function","title":"Example - ErrorMessage.im_a_teapot/2","doc":"iex> ErrorMessage.im_a_teapot(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :im_a_teapot, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#im_a_teapot/2-example"},{"type":"function","title":"ErrorMessage.insufficient_storage/1","doc":"Create insufficient_storage error message for status code 507","ref":"ErrorMessage.html#insufficient_storage/1"},{"type":"function","title":"Example - ErrorMessage.insufficient_storage/1","doc":"iex> ErrorMessage.insufficient_storage(\"error message\")\n    %ErrorMessage{code: :insufficient_storage, message: \"error message\"}","ref":"ErrorMessage.html#insufficient_storage/1-example"},{"type":"function","title":"ErrorMessage.insufficient_storage/2","doc":"Create insufficient_storage error message for status code 507 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#insufficient_storage/2"},{"type":"function","title":"Example - ErrorMessage.insufficient_storage/2","doc":"iex> ErrorMessage.insufficient_storage(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :insufficient_storage, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#insufficient_storage/2-example"},{"type":"function","title":"ErrorMessage.internal_server_error/1","doc":"Create internal_server_error error message for status code 500","ref":"ErrorMessage.html#internal_server_error/1"},{"type":"function","title":"Example - ErrorMessage.internal_server_error/1","doc":"iex> ErrorMessage.internal_server_error(\"error message\")\n    %ErrorMessage{code: :internal_server_error, message: \"error message\"}","ref":"ErrorMessage.html#internal_server_error/1-example"},{"type":"function","title":"ErrorMessage.internal_server_error/2","doc":"Create internal_server_error error message for status code 500 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#internal_server_error/2"},{"type":"function","title":"Example - ErrorMessage.internal_server_error/2","doc":"iex> ErrorMessage.internal_server_error(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :internal_server_error, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#internal_server_error/2-example"},{"type":"function","title":"ErrorMessage.length_required/1","doc":"Create length_required error message for status code 411","ref":"ErrorMessage.html#length_required/1"},{"type":"function","title":"Example - ErrorMessage.length_required/1","doc":"iex> ErrorMessage.length_required(\"error message\")\n    %ErrorMessage{code: :length_required, message: \"error message\"}","ref":"ErrorMessage.html#length_required/1-example"},{"type":"function","title":"ErrorMessage.length_required/2","doc":"Create length_required error message for status code 411 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#length_required/2"},{"type":"function","title":"Example - ErrorMessage.length_required/2","doc":"iex> ErrorMessage.length_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :length_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#length_required/2-example"},{"type":"function","title":"ErrorMessage.locked/1","doc":"Create locked error message for status code 423","ref":"ErrorMessage.html#locked/1"},{"type":"function","title":"Example - ErrorMessage.locked/1","doc":"iex> ErrorMessage.locked(\"error message\")\n    %ErrorMessage{code: :locked, message: \"error message\"}","ref":"ErrorMessage.html#locked/1-example"},{"type":"function","title":"ErrorMessage.locked/2","doc":"Create locked error message for status code 423 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#locked/2"},{"type":"function","title":"Example - ErrorMessage.locked/2","doc":"iex> ErrorMessage.locked(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :locked, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#locked/2-example"},{"type":"function","title":"ErrorMessage.loop_detected/1","doc":"Create loop_detected error message for status code 508","ref":"ErrorMessage.html#loop_detected/1"},{"type":"function","title":"Example - ErrorMessage.loop_detected/1","doc":"iex> ErrorMessage.loop_detected(\"error message\")\n    %ErrorMessage{code: :loop_detected, message: \"error message\"}","ref":"ErrorMessage.html#loop_detected/1-example"},{"type":"function","title":"ErrorMessage.loop_detected/2","doc":"Create loop_detected error message for status code 508 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#loop_detected/2"},{"type":"function","title":"Example - ErrorMessage.loop_detected/2","doc":"iex> ErrorMessage.loop_detected(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :loop_detected, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#loop_detected/2-example"},{"type":"function","title":"ErrorMessage.method_not_allowed/1","doc":"Create method_not_allowed error message for status code 405","ref":"ErrorMessage.html#method_not_allowed/1"},{"type":"function","title":"Example - ErrorMessage.method_not_allowed/1","doc":"iex> ErrorMessage.method_not_allowed(\"error message\")\n    %ErrorMessage{code: :method_not_allowed, message: \"error message\"}","ref":"ErrorMessage.html#method_not_allowed/1-example"},{"type":"function","title":"ErrorMessage.method_not_allowed/2","doc":"Create method_not_allowed error message for status code 405 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#method_not_allowed/2"},{"type":"function","title":"Example - ErrorMessage.method_not_allowed/2","doc":"iex> ErrorMessage.method_not_allowed(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :method_not_allowed, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#method_not_allowed/2-example"},{"type":"function","title":"ErrorMessage.misdirected_request/1","doc":"Create misdirected_request error message for status code 421","ref":"ErrorMessage.html#misdirected_request/1"},{"type":"function","title":"Example - ErrorMessage.misdirected_request/1","doc":"iex> ErrorMessage.misdirected_request(\"error message\")\n    %ErrorMessage{code: :misdirected_request, message: \"error message\"}","ref":"ErrorMessage.html#misdirected_request/1-example"},{"type":"function","title":"ErrorMessage.misdirected_request/2","doc":"Create misdirected_request error message for status code 421 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#misdirected_request/2"},{"type":"function","title":"Example - ErrorMessage.misdirected_request/2","doc":"iex> ErrorMessage.misdirected_request(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :misdirected_request, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#misdirected_request/2-example"},{"type":"function","title":"ErrorMessage.moved_permanently/1","doc":"Create moved_permanently error message for status code 301","ref":"ErrorMessage.html#moved_permanently/1"},{"type":"function","title":"Example - ErrorMessage.moved_permanently/1","doc":"iex> ErrorMessage.moved_permanently(\"error message\")\n    %ErrorMessage{code: :moved_permanently, message: \"error message\"}","ref":"ErrorMessage.html#moved_permanently/1-example"},{"type":"function","title":"ErrorMessage.moved_permanently/2","doc":"Create moved_permanently error message for status code 301 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#moved_permanently/2"},{"type":"function","title":"Example - ErrorMessage.moved_permanently/2","doc":"iex> ErrorMessage.moved_permanently(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :moved_permanently, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#moved_permanently/2-example"},{"type":"function","title":"ErrorMessage.multiple_choices/1","doc":"Create multiple_choices error message for status code 300","ref":"ErrorMessage.html#multiple_choices/1"},{"type":"function","title":"Example - ErrorMessage.multiple_choices/1","doc":"iex> ErrorMessage.multiple_choices(\"error message\")\n    %ErrorMessage{code: :multiple_choices, message: \"error message\"}","ref":"ErrorMessage.html#multiple_choices/1-example"},{"type":"function","title":"ErrorMessage.multiple_choices/2","doc":"Create multiple_choices error message for status code 300 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#multiple_choices/2"},{"type":"function","title":"Example - ErrorMessage.multiple_choices/2","doc":"iex> ErrorMessage.multiple_choices(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :multiple_choices, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#multiple_choices/2-example"},{"type":"function","title":"ErrorMessage.network_authentication_required/1","doc":"Create network_authentication_required error message for status code 511","ref":"ErrorMessage.html#network_authentication_required/1"},{"type":"function","title":"Example - ErrorMessage.network_authentication_required/1","doc":"iex> ErrorMessage.network_authentication_required(\"error message\")\n    %ErrorMessage{code: :network_authentication_required, message: \"error message\"}","ref":"ErrorMessage.html#network_authentication_required/1-example"},{"type":"function","title":"ErrorMessage.network_authentication_required/2","doc":"Create network_authentication_required error message for status code 511 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#network_authentication_required/2"},{"type":"function","title":"Example - ErrorMessage.network_authentication_required/2","doc":"iex> ErrorMessage.network_authentication_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :network_authentication_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#network_authentication_required/2-example"},{"type":"function","title":"ErrorMessage.not_acceptable/1","doc":"Create not_acceptable error message for status code 406","ref":"ErrorMessage.html#not_acceptable/1"},{"type":"function","title":"Example - ErrorMessage.not_acceptable/1","doc":"iex> ErrorMessage.not_acceptable(\"error message\")\n    %ErrorMessage{code: :not_acceptable, message: \"error message\"}","ref":"ErrorMessage.html#not_acceptable/1-example"},{"type":"function","title":"ErrorMessage.not_acceptable/2","doc":"Create not_acceptable error message for status code 406 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#not_acceptable/2"},{"type":"function","title":"Example - ErrorMessage.not_acceptable/2","doc":"iex> ErrorMessage.not_acceptable(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :not_acceptable, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#not_acceptable/2-example"},{"type":"function","title":"ErrorMessage.not_extended/1","doc":"Create not_extended error message for status code 510","ref":"ErrorMessage.html#not_extended/1"},{"type":"function","title":"Example - ErrorMessage.not_extended/1","doc":"iex> ErrorMessage.not_extended(\"error message\")\n    %ErrorMessage{code: :not_extended, message: \"error message\"}","ref":"ErrorMessage.html#not_extended/1-example"},{"type":"function","title":"ErrorMessage.not_extended/2","doc":"Create not_extended error message for status code 510 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#not_extended/2"},{"type":"function","title":"Example - ErrorMessage.not_extended/2","doc":"iex> ErrorMessage.not_extended(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :not_extended, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#not_extended/2-example"},{"type":"function","title":"ErrorMessage.not_found/1","doc":"Create not_found error message for status code 404","ref":"ErrorMessage.html#not_found/1"},{"type":"function","title":"Example - ErrorMessage.not_found/1","doc":"iex> ErrorMessage.not_found(\"error message\")\n    %ErrorMessage{code: :not_found, message: \"error message\"}","ref":"ErrorMessage.html#not_found/1-example"},{"type":"function","title":"ErrorMessage.not_found/2","doc":"Create not_found error message for status code 404 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#not_found/2"},{"type":"function","title":"Example - ErrorMessage.not_found/2","doc":"iex> ErrorMessage.not_found(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :not_found, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#not_found/2-example"},{"type":"function","title":"ErrorMessage.not_implemented/1","doc":"Create not_implemented error message for status code 501","ref":"ErrorMessage.html#not_implemented/1"},{"type":"function","title":"Example - ErrorMessage.not_implemented/1","doc":"iex> ErrorMessage.not_implemented(\"error message\")\n    %ErrorMessage{code: :not_implemented, message: \"error message\"}","ref":"ErrorMessage.html#not_implemented/1-example"},{"type":"function","title":"ErrorMessage.not_implemented/2","doc":"Create not_implemented error message for status code 501 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#not_implemented/2"},{"type":"function","title":"Example - ErrorMessage.not_implemented/2","doc":"iex> ErrorMessage.not_implemented(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :not_implemented, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#not_implemented/2-example"},{"type":"function","title":"ErrorMessage.not_modified/1","doc":"Create not_modified error message for status code 304","ref":"ErrorMessage.html#not_modified/1"},{"type":"function","title":"Example - ErrorMessage.not_modified/1","doc":"iex> ErrorMessage.not_modified(\"error message\")\n    %ErrorMessage{code: :not_modified, message: \"error message\"}","ref":"ErrorMessage.html#not_modified/1-example"},{"type":"function","title":"ErrorMessage.not_modified/2","doc":"Create not_modified error message for status code 304 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#not_modified/2"},{"type":"function","title":"Example - ErrorMessage.not_modified/2","doc":"iex> ErrorMessage.not_modified(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :not_modified, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#not_modified/2-example"},{"type":"function","title":"ErrorMessage.payment_required/1","doc":"Create payment_required error message for status code 402","ref":"ErrorMessage.html#payment_required/1"},{"type":"function","title":"Example - ErrorMessage.payment_required/1","doc":"iex> ErrorMessage.payment_required(\"error message\")\n    %ErrorMessage{code: :payment_required, message: \"error message\"}","ref":"ErrorMessage.html#payment_required/1-example"},{"type":"function","title":"ErrorMessage.payment_required/2","doc":"Create payment_required error message for status code 402 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#payment_required/2"},{"type":"function","title":"Example - ErrorMessage.payment_required/2","doc":"iex> ErrorMessage.payment_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :payment_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#payment_required/2-example"},{"type":"function","title":"ErrorMessage.permanent_redirect/1","doc":"Create permanent_redirect error message for status code 308","ref":"ErrorMessage.html#permanent_redirect/1"},{"type":"function","title":"Example - ErrorMessage.permanent_redirect/1","doc":"iex> ErrorMessage.permanent_redirect(\"error message\")\n    %ErrorMessage{code: :permanent_redirect, message: \"error message\"}","ref":"ErrorMessage.html#permanent_redirect/1-example"},{"type":"function","title":"ErrorMessage.permanent_redirect/2","doc":"Create permanent_redirect error message for status code 308 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#permanent_redirect/2"},{"type":"function","title":"Example - ErrorMessage.permanent_redirect/2","doc":"iex> ErrorMessage.permanent_redirect(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :permanent_redirect, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#permanent_redirect/2-example"},{"type":"function","title":"ErrorMessage.precondition_failed/1","doc":"Create precondition_failed error message for status code 412","ref":"ErrorMessage.html#precondition_failed/1"},{"type":"function","title":"Example - ErrorMessage.precondition_failed/1","doc":"iex> ErrorMessage.precondition_failed(\"error message\")\n    %ErrorMessage{code: :precondition_failed, message: \"error message\"}","ref":"ErrorMessage.html#precondition_failed/1-example"},{"type":"function","title":"ErrorMessage.precondition_failed/2","doc":"Create precondition_failed error message for status code 412 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#precondition_failed/2"},{"type":"function","title":"Example - ErrorMessage.precondition_failed/2","doc":"iex> ErrorMessage.precondition_failed(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :precondition_failed, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#precondition_failed/2-example"},{"type":"function","title":"ErrorMessage.precondition_required/1","doc":"Create precondition_required error message for status code 428","ref":"ErrorMessage.html#precondition_required/1"},{"type":"function","title":"Example - ErrorMessage.precondition_required/1","doc":"iex> ErrorMessage.precondition_required(\"error message\")\n    %ErrorMessage{code: :precondition_required, message: \"error message\"}","ref":"ErrorMessage.html#precondition_required/1-example"},{"type":"function","title":"ErrorMessage.precondition_required/2","doc":"Create precondition_required error message for status code 428 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#precondition_required/2"},{"type":"function","title":"Example - ErrorMessage.precondition_required/2","doc":"iex> ErrorMessage.precondition_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :precondition_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#precondition_required/2-example"},{"type":"function","title":"ErrorMessage.proxy_authentication_required/1","doc":"Create proxy_authentication_required error message for status code 407","ref":"ErrorMessage.html#proxy_authentication_required/1"},{"type":"function","title":"Example - ErrorMessage.proxy_authentication_required/1","doc":"iex> ErrorMessage.proxy_authentication_required(\"error message\")\n    %ErrorMessage{code: :proxy_authentication_required, message: \"error message\"}","ref":"ErrorMessage.html#proxy_authentication_required/1-example"},{"type":"function","title":"ErrorMessage.proxy_authentication_required/2","doc":"Create proxy_authentication_required error message for status code 407 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#proxy_authentication_required/2"},{"type":"function","title":"Example - ErrorMessage.proxy_authentication_required/2","doc":"iex> ErrorMessage.proxy_authentication_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :proxy_authentication_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#proxy_authentication_required/2-example"},{"type":"function","title":"ErrorMessage.request_entity_too_large/1","doc":"Create request_entity_too_large error message for status code 413","ref":"ErrorMessage.html#request_entity_too_large/1"},{"type":"function","title":"Example - ErrorMessage.request_entity_too_large/1","doc":"iex> ErrorMessage.request_entity_too_large(\"error message\")\n    %ErrorMessage{code: :request_entity_too_large, message: \"error message\"}","ref":"ErrorMessage.html#request_entity_too_large/1-example"},{"type":"function","title":"ErrorMessage.request_entity_too_large/2","doc":"Create request_entity_too_large error message for status code 413 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#request_entity_too_large/2"},{"type":"function","title":"Example - ErrorMessage.request_entity_too_large/2","doc":"iex> ErrorMessage.request_entity_too_large(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :request_entity_too_large, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#request_entity_too_large/2-example"},{"type":"function","title":"ErrorMessage.request_header_fields_too_large/1","doc":"Create request_header_fields_too_large error message for status code 431","ref":"ErrorMessage.html#request_header_fields_too_large/1"},{"type":"function","title":"Example - ErrorMessage.request_header_fields_too_large/1","doc":"iex> ErrorMessage.request_header_fields_too_large(\"error message\")\n    %ErrorMessage{code: :request_header_fields_too_large, message: \"error message\"}","ref":"ErrorMessage.html#request_header_fields_too_large/1-example"},{"type":"function","title":"ErrorMessage.request_header_fields_too_large/2","doc":"Create request_header_fields_too_large error message for status code 431 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#request_header_fields_too_large/2"},{"type":"function","title":"Example - ErrorMessage.request_header_fields_too_large/2","doc":"iex> ErrorMessage.request_header_fields_too_large(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :request_header_fields_too_large, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#request_header_fields_too_large/2-example"},{"type":"function","title":"ErrorMessage.request_timeout/1","doc":"Create request_timeout error message for status code 408","ref":"ErrorMessage.html#request_timeout/1"},{"type":"function","title":"Example - ErrorMessage.request_timeout/1","doc":"iex> ErrorMessage.request_timeout(\"error message\")\n    %ErrorMessage{code: :request_timeout, message: \"error message\"}","ref":"ErrorMessage.html#request_timeout/1-example"},{"type":"function","title":"ErrorMessage.request_timeout/2","doc":"Create request_timeout error message for status code 408 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#request_timeout/2"},{"type":"function","title":"Example - ErrorMessage.request_timeout/2","doc":"iex> ErrorMessage.request_timeout(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :request_timeout, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#request_timeout/2-example"},{"type":"function","title":"ErrorMessage.request_uri_too_long/1","doc":"Create request_uri_too_long error message for status code 414","ref":"ErrorMessage.html#request_uri_too_long/1"},{"type":"function","title":"Example - ErrorMessage.request_uri_too_long/1","doc":"iex> ErrorMessage.request_uri_too_long(\"error message\")\n    %ErrorMessage{code: :request_uri_too_long, message: \"error message\"}","ref":"ErrorMessage.html#request_uri_too_long/1-example"},{"type":"function","title":"ErrorMessage.request_uri_too_long/2","doc":"Create request_uri_too_long error message for status code 414 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#request_uri_too_long/2"},{"type":"function","title":"Example - ErrorMessage.request_uri_too_long/2","doc":"iex> ErrorMessage.request_uri_too_long(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :request_uri_too_long, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#request_uri_too_long/2-example"},{"type":"function","title":"ErrorMessage.requested_range_not_satisfiable/1","doc":"Create requested_range_not_satisfiable error message for status code 416","ref":"ErrorMessage.html#requested_range_not_satisfiable/1"},{"type":"function","title":"Example - ErrorMessage.requested_range_not_satisfiable/1","doc":"iex> ErrorMessage.requested_range_not_satisfiable(\"error message\")\n    %ErrorMessage{code: :requested_range_not_satisfiable, message: \"error message\"}","ref":"ErrorMessage.html#requested_range_not_satisfiable/1-example"},{"type":"function","title":"ErrorMessage.requested_range_not_satisfiable/2","doc":"Create requested_range_not_satisfiable error message for status code 416 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#requested_range_not_satisfiable/2"},{"type":"function","title":"Example - ErrorMessage.requested_range_not_satisfiable/2","doc":"iex> ErrorMessage.requested_range_not_satisfiable(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :requested_range_not_satisfiable, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#requested_range_not_satisfiable/2-example"},{"type":"function","title":"ErrorMessage.see_other/1","doc":"Create see_other error message for status code 303","ref":"ErrorMessage.html#see_other/1"},{"type":"function","title":"Example - ErrorMessage.see_other/1","doc":"iex> ErrorMessage.see_other(\"error message\")\n    %ErrorMessage{code: :see_other, message: \"error message\"}","ref":"ErrorMessage.html#see_other/1-example"},{"type":"function","title":"ErrorMessage.see_other/2","doc":"Create see_other error message for status code 303 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#see_other/2"},{"type":"function","title":"Example - ErrorMessage.see_other/2","doc":"iex> ErrorMessage.see_other(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :see_other, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#see_other/2-example"},{"type":"function","title":"ErrorMessage.service_unavailable/1","doc":"Create service_unavailable error message for status code 503","ref":"ErrorMessage.html#service_unavailable/1"},{"type":"function","title":"Example - ErrorMessage.service_unavailable/1","doc":"iex> ErrorMessage.service_unavailable(\"error message\")\n    %ErrorMessage{code: :service_unavailable, message: \"error message\"}","ref":"ErrorMessage.html#service_unavailable/1-example"},{"type":"function","title":"ErrorMessage.service_unavailable/2","doc":"Create service_unavailable error message for status code 503 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#service_unavailable/2"},{"type":"function","title":"Example - ErrorMessage.service_unavailable/2","doc":"iex> ErrorMessage.service_unavailable(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :service_unavailable, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#service_unavailable/2-example"},{"type":"function","title":"ErrorMessage.switch_proxy/1","doc":"Create switch_proxy error message for status code 306","ref":"ErrorMessage.html#switch_proxy/1"},{"type":"function","title":"Example - ErrorMessage.switch_proxy/1","doc":"iex> ErrorMessage.switch_proxy(\"error message\")\n    %ErrorMessage{code: :switch_proxy, message: \"error message\"}","ref":"ErrorMessage.html#switch_proxy/1-example"},{"type":"function","title":"ErrorMessage.switch_proxy/2","doc":"Create switch_proxy error message for status code 306 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#switch_proxy/2"},{"type":"function","title":"Example - ErrorMessage.switch_proxy/2","doc":"iex> ErrorMessage.switch_proxy(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :switch_proxy, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#switch_proxy/2-example"},{"type":"type","title":"ErrorMessage.t/0","doc":"","ref":"ErrorMessage.html#t:t/0"},{"type":"type","title":"ErrorMessage.t/1","doc":"","ref":"ErrorMessage.html#t:t/1"},{"type":"type","title":"ErrorMessage.t_map/0","doc":"","ref":"ErrorMessage.html#t:t_map/0"},{"type":"type","title":"ErrorMessage.t_map/1","doc":"","ref":"ErrorMessage.html#t:t_map/1"},{"type":"type","title":"ErrorMessage.t_ok_res/0","doc":"","ref":"ErrorMessage.html#t:t_ok_res/0"},{"type":"type","title":"ErrorMessage.t_ok_res/1","doc":"","ref":"ErrorMessage.html#t:t_ok_res/1"},{"type":"type","title":"ErrorMessage.t_res/0","doc":"","ref":"ErrorMessage.html#t:t_res/0"},{"type":"type","title":"ErrorMessage.t_res/1","doc":"","ref":"ErrorMessage.html#t:t_res/1"},{"type":"type","title":"ErrorMessage.t_res/2","doc":"","ref":"ErrorMessage.html#t:t_res/2"},{"type":"function","title":"ErrorMessage.temporary_redirect/1","doc":"Create temporary_redirect error message for status code 307","ref":"ErrorMessage.html#temporary_redirect/1"},{"type":"function","title":"Example - ErrorMessage.temporary_redirect/1","doc":"iex> ErrorMessage.temporary_redirect(\"error message\")\n    %ErrorMessage{code: :temporary_redirect, message: \"error message\"}","ref":"ErrorMessage.html#temporary_redirect/1-example"},{"type":"function","title":"ErrorMessage.temporary_redirect/2","doc":"Create temporary_redirect error message for status code 307 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#temporary_redirect/2"},{"type":"function","title":"Example - ErrorMessage.temporary_redirect/2","doc":"iex> ErrorMessage.temporary_redirect(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :temporary_redirect, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#temporary_redirect/2-example"},{"type":"function","title":"ErrorMessage.to_jsonable_map/1","doc":"Converts an `%ErrorMessage{}` struct to a map and makes sure that the\ncontents of the details map can be converted to json","ref":"ErrorMessage.html#to_jsonable_map/1"},{"type":"function","title":"Example - ErrorMessage.to_jsonable_map/1","doc":"iex> ErrorMessage.to_jsonable_map(ErrorMessage.not_found(\"couldn't find user\", %{user_id: \"as21fasdfJ\"}))\n      %{code: :not_found, message: \"couldn't find user\", details: %{user_id: \"as21fasdfJ\"}}\n\n      iex> error = ErrorMessage.im_a_teapot(\"teapot\", %{\n      ...>   user: %{health: {:alive, 500}},\n      ...>   test: %TestStruct{a: [Date.new!(2020, 1, 10)]}\n      ...> })\n      iex> ErrorMessage.to_jsonable_map(error)\n      %{\n        code: :im_a_teapot,\n        message: \"teapot\",\n        details: %{\n          user: %{health: [:alive, 500]},\n          test: %{struct: \"ErrorMessageTest.TestStruct\", data: %{a: [\"2020-01-10\"]}}\n        }\n      }","ref":"ErrorMessage.html#to_jsonable_map/1-example"},{"type":"function","title":"ErrorMessage.to_string/1","doc":"Converts an `%ErrorMessage{}` struct to a string formatted error message","ref":"ErrorMessage.html#to_string/1"},{"type":"function","title":"Example - ErrorMessage.to_string/1","doc":"iex> ErrorMessage.to_string(ErrorMessage.internal_server_error(\"Something bad happened\", %{result: :unknown}))\n      \"internal_server_error - Something bad happened\\nDetails: \\n%{result: :unknown}\"","ref":"ErrorMessage.html#to_string/1-example"},{"type":"function","title":"ErrorMessage.too_early/1","doc":"Create too_early error message for status code 425","ref":"ErrorMessage.html#too_early/1"},{"type":"function","title":"Example - ErrorMessage.too_early/1","doc":"iex> ErrorMessage.too_early(\"error message\")\n    %ErrorMessage{code: :too_early, message: \"error message\"}","ref":"ErrorMessage.html#too_early/1-example"},{"type":"function","title":"ErrorMessage.too_early/2","doc":"Create too_early error message for status code 425 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#too_early/2"},{"type":"function","title":"Example - ErrorMessage.too_early/2","doc":"iex> ErrorMessage.too_early(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :too_early, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#too_early/2-example"},{"type":"function","title":"ErrorMessage.too_many_requests/1","doc":"Create too_many_requests error message for status code 429","ref":"ErrorMessage.html#too_many_requests/1"},{"type":"function","title":"Example - ErrorMessage.too_many_requests/1","doc":"iex> ErrorMessage.too_many_requests(\"error message\")\n    %ErrorMessage{code: :too_many_requests, message: \"error message\"}","ref":"ErrorMessage.html#too_many_requests/1-example"},{"type":"function","title":"ErrorMessage.too_many_requests/2","doc":"Create too_many_requests error message for status code 429 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#too_many_requests/2"},{"type":"function","title":"Example - ErrorMessage.too_many_requests/2","doc":"iex> ErrorMessage.too_many_requests(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :too_many_requests, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#too_many_requests/2-example"},{"type":"function","title":"ErrorMessage.unauthorized/1","doc":"Create unauthorized error message for status code 401","ref":"ErrorMessage.html#unauthorized/1"},{"type":"function","title":"Example - ErrorMessage.unauthorized/1","doc":"iex> ErrorMessage.unauthorized(\"error message\")\n    %ErrorMessage{code: :unauthorized, message: \"error message\"}","ref":"ErrorMessage.html#unauthorized/1-example"},{"type":"function","title":"ErrorMessage.unauthorized/2","doc":"Create unauthorized error message for status code 401 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#unauthorized/2"},{"type":"function","title":"Example - ErrorMessage.unauthorized/2","doc":"iex> ErrorMessage.unauthorized(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :unauthorized, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#unauthorized/2-example"},{"type":"function","title":"ErrorMessage.unavailable_for_legal_reasons/1","doc":"Create unavailable_for_legal_reasons error message for status code 451","ref":"ErrorMessage.html#unavailable_for_legal_reasons/1"},{"type":"function","title":"Example - ErrorMessage.unavailable_for_legal_reasons/1","doc":"iex> ErrorMessage.unavailable_for_legal_reasons(\"error message\")\n    %ErrorMessage{code: :unavailable_for_legal_reasons, message: \"error message\"}","ref":"ErrorMessage.html#unavailable_for_legal_reasons/1-example"},{"type":"function","title":"ErrorMessage.unavailable_for_legal_reasons/2","doc":"Create unavailable_for_legal_reasons error message for status code 451 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#unavailable_for_legal_reasons/2"},{"type":"function","title":"Example - ErrorMessage.unavailable_for_legal_reasons/2","doc":"iex> ErrorMessage.unavailable_for_legal_reasons(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :unavailable_for_legal_reasons, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#unavailable_for_legal_reasons/2-example"},{"type":"function","title":"ErrorMessage.unprocessable_entity/1","doc":"Create unprocessable_entity error message for status code 422","ref":"ErrorMessage.html#unprocessable_entity/1"},{"type":"function","title":"Example - ErrorMessage.unprocessable_entity/1","doc":"iex> ErrorMessage.unprocessable_entity(\"error message\")\n    %ErrorMessage{code: :unprocessable_entity, message: \"error message\"}","ref":"ErrorMessage.html#unprocessable_entity/1-example"},{"type":"function","title":"ErrorMessage.unprocessable_entity/2","doc":"Create unprocessable_entity error message for status code 422 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#unprocessable_entity/2"},{"type":"function","title":"Example - ErrorMessage.unprocessable_entity/2","doc":"iex> ErrorMessage.unprocessable_entity(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :unprocessable_entity, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#unprocessable_entity/2-example"},{"type":"function","title":"ErrorMessage.unsupported_media_type/1","doc":"Create unsupported_media_type error message for status code 415","ref":"ErrorMessage.html#unsupported_media_type/1"},{"type":"function","title":"Example - ErrorMessage.unsupported_media_type/1","doc":"iex> ErrorMessage.unsupported_media_type(\"error message\")\n    %ErrorMessage{code: :unsupported_media_type, message: \"error message\"}","ref":"ErrorMessage.html#unsupported_media_type/1-example"},{"type":"function","title":"ErrorMessage.unsupported_media_type/2","doc":"Create unsupported_media_type error message for status code 415 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#unsupported_media_type/2"},{"type":"function","title":"Example - ErrorMessage.unsupported_media_type/2","doc":"iex> ErrorMessage.unsupported_media_type(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :unsupported_media_type, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#unsupported_media_type/2-example"},{"type":"function","title":"ErrorMessage.upgrade_required/1","doc":"Create upgrade_required error message for status code 426","ref":"ErrorMessage.html#upgrade_required/1"},{"type":"function","title":"Example - ErrorMessage.upgrade_required/1","doc":"iex> ErrorMessage.upgrade_required(\"error message\")\n    %ErrorMessage{code: :upgrade_required, message: \"error message\"}","ref":"ErrorMessage.html#upgrade_required/1-example"},{"type":"function","title":"ErrorMessage.upgrade_required/2","doc":"Create upgrade_required error message for status code 426 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#upgrade_required/2"},{"type":"function","title":"Example - ErrorMessage.upgrade_required/2","doc":"iex> ErrorMessage.upgrade_required(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :upgrade_required, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#upgrade_required/2-example"},{"type":"function","title":"ErrorMessage.use_proxy/1","doc":"Create use_proxy error message for status code 305","ref":"ErrorMessage.html#use_proxy/1"},{"type":"function","title":"Example - ErrorMessage.use_proxy/1","doc":"iex> ErrorMessage.use_proxy(\"error message\")\n    %ErrorMessage{code: :use_proxy, message: \"error message\"}","ref":"ErrorMessage.html#use_proxy/1-example"},{"type":"function","title":"ErrorMessage.use_proxy/2","doc":"Create use_proxy error message for status code 305 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#use_proxy/2"},{"type":"function","title":"Example - ErrorMessage.use_proxy/2","doc":"iex> ErrorMessage.use_proxy(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :use_proxy, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#use_proxy/2-example"},{"type":"function","title":"ErrorMessage.variant_also_negotiates/1","doc":"Create variant_also_negotiates error message for status code 506","ref":"ErrorMessage.html#variant_also_negotiates/1"},{"type":"function","title":"Example - ErrorMessage.variant_also_negotiates/1","doc":"iex> ErrorMessage.variant_also_negotiates(\"error message\")\n    %ErrorMessage{code: :variant_also_negotiates, message: \"error message\"}","ref":"ErrorMessage.html#variant_also_negotiates/1-example"},{"type":"function","title":"ErrorMessage.variant_also_negotiates/2","doc":"Create variant_also_negotiates error message for status code 506 with a details item which is\npassed in as the details key under the `ErrorMessage` struct","ref":"ErrorMessage.html#variant_also_negotiates/2"},{"type":"function","title":"Example - ErrorMessage.variant_also_negotiates/2","doc":"iex> ErrorMessage.variant_also_negotiates(\"error message\", %{item: 1234})\n    %ErrorMessage{code: :variant_also_negotiates, message: \"error message\", details: %{item: 1234}}","ref":"ErrorMessage.html#variant_also_negotiates/2-example"},{"type":"extras","title":"ErrorMessage","doc":"# ErrorMessage\n\nErrorMessage is a library designed to simplify error handling in Elixir applications by providing a consistent, HTTP-inspired error system. It creates a unified approach to error representation, making your code more predictable and maintainable.","ref":"overview.html"},{"type":"extras","title":"Key Features - ErrorMessage","doc":"- **Consistent Error Structure**: All errors follow the same format with code, message, and optional details\n- **HTTP Status Code Integration**: Uses standard HTTP status codes as error codes\n- **JSON Serialization**: Easy conversion to JSON for API responses\n- **String Formatting**: Implementation of String.Chars protocol for easy logging\n- **Type Specifications**: Comprehensive typespecs for better static analysis","ref":"overview.html#key-features"},{"type":"extras","title":"Documentation Structure - ErrorMessage","doc":"1. **Tutorials**: Step-by-step lessons to help you get started with ErrorMessage\n2. **How-To Guides**: Practical guides for solving specific problems\n3. **Explanation**: Conceptual discussions about ErrorMessage's design and principles\n4. **Reference**: Technical descriptions of ErrorMessage's modules, functions, and types","ref":"overview.html#documentation-structure"},{"type":"extras","title":"Installation - ErrorMessage","doc":"Add `error_message` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:error_message, \"~> 0.3.2\"}\n  ]\nend\n```","ref":"overview.html#installation"},{"type":"extras","title":"Basic Usage - ErrorMessage","doc":"```elixir\n# Create a not_found error with a message and details\nerror = ErrorMessage.not_found(\"User not found\", %{user_id: 123})\n# %ErrorMessage{code: :not_found, message: \"User not found\", details: %{user_id: 123}}\n\n# Get the HTTP status code\nErrorMessage.http_code(error)  # Returns 404\n\n# Convert to string for logging\nto_string(error)  # Returns \"not_found - User not found\\nDetails: \\n%{user_id: 123}\"\n\n# Convert to a map for JSON serialization\nErrorMessage.to_jsonable_map(error)\n# %{code: :not_found, message: \"User not found\", details: %{user_id: 123}}\n```","ref":"overview.html#basic-usage"},{"type":"extras","title":"Why Use ErrorMessage? - ErrorMessage","doc":"ErrorMessage helps you build more resilient applications by:\n\n1. Providing a consistent error interface across your entire application\n2. Making error handling more predictable and pattern-matchable\n3. Simplifying API responses with proper HTTP status codes\n4. Improving logging with meaningful error messages\n5. Ensuring error details are properly serialized for various formats\n\nCheck out the rest of the documentation to learn more about how to use ErrorMessage effectively in your Elixir applications.","ref":"overview.html#why-use-errormessage"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"0.3.3 - CHANGELOG","doc":"- Add docs","ref":"changelog.html#0-3-3"},{"type":"extras","title":"0.3.2 - CHANGELOG","doc":"- Add `t_ok_res` type for `:ok | {:error, ErrorMessage.t}`","ref":"changelog.html#0-3-2"},{"type":"extras","title":"0.3.1 - CHANGELOG","doc":"- Add ability to serialize non local pids as well","ref":"changelog.html#0-3-1"},{"type":"extras","title":"0.3.0 - CHANGELOG","doc":"- Remove the inspect protocol impl so it uses default inspect","ref":"changelog.html#0-3-0"},{"type":"extras","title":"0.2.4 - CHANGELOG","doc":"- Add ability to encode PIDs in error messages and pull out their registered name","ref":"changelog.html#0-2-4"},{"type":"extras","title":"0.2.3 - CHANGELOG","doc":"- Add `http_code_reason_atom`","ref":"changelog.html#0-2-3"},{"type":"extras","title":"0.2.2 - CHANGELOG","doc":"- Don't generate details: nil in outputs anymore","ref":"changelog.html#0-2-2"},{"type":"extras","title":"0.2.1 - CHANGELOG","doc":"- Add `t_res` types","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.2.0 - CHANGELOG","doc":"- add `http_code` function\n- add http codes to docs\n- add request_id to jsonable_map when possible","ref":"changelog.html#0-2-0"},{"type":"extras","title":"0.1.4 - CHANGELOG","doc":"- Make details pretty print on inspect","ref":"changelog.html#0-1-4"},{"type":"extras","title":"0.1.3 - CHANGELOG","doc":"- Change message typing to String.t","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.2 - CHANGELOG","doc":"- Add better typing for specs","ref":"changelog.html#0-1-2"},{"type":"extras","title":"0.1.1 - CHANGELOG","doc":"- Add support for Jason @derive when library is present","ref":"changelog.html#0-1-1"},{"type":"extras","title":"0.1.0 - CHANGELOG","doc":"- Initial Release","ref":"changelog.html#0-1-0"},{"type":"extras","title":"Building an Error Handling Workflow","doc":"# Building an Error Handling Workflow\n\nThis tutorial will guide you through creating a complete error handling workflow using ErrorMessage in an Elixir application. We'll build a simple user management system that demonstrates how to use ErrorMessage throughout different layers of your application.","ref":"error_handling_workflow.html"},{"type":"extras","title":"Prerequisites - Building an Error Handling Workflow","doc":"- Basic knowledge of Elixir\n- ErrorMessage library installed\n- Basic understanding of Ecto (optional, for the database example)","ref":"error_handling_workflow.html#prerequisites"},{"type":"extras","title":"Step 1: Define Your Domain Logic - Building an Error Handling Workflow","doc":"Let's start by defining a simple user management module:\n\n```elixir\ndefmodule MyApp.Users do\n  @moduledoc \"\"\"\n  User management functionality\n  \"\"\"\n  \n  # In-memory user database for this example\n  @users %{\n    1 => %{id: 1, name: \"Alice\", email: \"alice@example.com\", role: :admin},\n    2 => %{id: 2, name: \"Bob\", email: \"bob@example.com\", role: :user}\n  }\n  \n  @doc \"\"\"\n  Get a user by ID\n  \"\"\"\n  @spec get_user(integer()) :: {:ok, map()} | {:error, ErrorMessage.t()}\n  def get_user(id) when is_integer(id) do\n    case Map.get(@users, id) do\n      nil -> \n        {:error, ErrorMessage.not_found(\"User not found\", %{user_id: id})}\n      user -> \n        {:ok, user}\n    end\n  end\n  \n  @doc \"\"\"\n  Get a user by email\n  \"\"\"\n  @spec get_user_by_email(String.t()) :: {:ok, map()} | {:error, ErrorMessage.t()}\n  def get_user_by_email(email) when is_binary(email) do\n    user = Enum.find_value(@users, fn {_id, user} -> \n      if user.email == email, do: user\n    end)\n    \n    case user do\n      nil -> \n        {:error, ErrorMessage.not_found(\"User not found\", %{email: email})}\n      user -> \n        {:ok, user}\n    end\n  end\n  \n  @doc \"\"\"\n  Update a user\n  \"\"\"\n  @spec update_user(integer(), map()) :: {:ok, map()} | {:error, ErrorMessage.t()}\n  def update_user(id, params) when is_integer(id) and is_map(params) do\n    with {:ok, user} <- get_user(id),\n         {:ok, updated_user} <- validate_update(user, params) do\n      # In a real app, we would persist the changes\n      {:ok, updated_user}\n    end\n  end\n  \n  defp validate_update(user, %{role: role} = params) do\n    case role do\n      role when role in [:admin, :user] ->\n        {:ok, Map.merge(user, Map.take(params, [:name, :email, :role]))}\n      _ ->\n        {:error, ErrorMessage.unprocessable_entity(\"Invalid role\", %{\n          allowed_roles: [:admin, :user],\n          provided_role: role\n        })}\n    end\n  end\n  \n  defp validate_update(user, params) do\n    {:ok, Map.merge(user, Map.take(params, [:name, :email]))}\n  end\nend\n```","ref":"error_handling_workflow.html#step-1-define-your-domain-logic"},{"type":"extras","title":"Step 2: Create a Service Layer - Building an Error Handling Workflow","doc":"Now, let's create a service layer that uses our domain logic and adds additional business rules:\n\n```elixir\ndefmodule MyApp.UserService do\n  @moduledoc \"\"\"\n  User service with business logic\n  \"\"\"\n  \n  alias MyApp.Users\n  \n  @doc \"\"\"\n  Get user profile with authorization check\n  \"\"\"\n  @spec get_profile(integer(), map()) :: {:ok, map()} | {:error, ErrorMessage.t()}\n  def get_profile(user_id, current_user) do\n    with {:ok, user} <- Users.get_user(user_id),\n         :ok <- authorize_profile_access(user, current_user) do\n      {:ok, format_profile(user)}\n    end\n  end\n  \n  @doc \"\"\"\n  Update user with authorization check\n  \"\"\"\n  @spec update_profile(integer(), map(), map()) :: {:ok, map()} | {:error, ErrorMessage.t()}\n  def update_profile(user_id, params, current_user) do\n    with {:ok, user} <- Users.get_user(user_id),\n         :ok <- authorize_profile_update(user, current_user),\n         {:ok, updated_user} <- Users.update_user(user_id, params) do\n      {:ok, format_profile(updated_user)}\n    end\n  end\n  \n  # Authorization helpers\n  \n  defp authorize_profile_access(user, current_user) do\n    cond do\n      current_user.id == user.id -> :ok\n      current_user.role == :admin -> :ok\n      true -> {:error, ErrorMessage.forbidden(\"Not authorized to view this profile\")}\n    end\n  end\n  \n  defp authorize_profile_update(user, current_user) do\n    cond do\n      current_user.id == user.id -> :ok\n      current_user.role == :admin -> :ok\n      true -> {:error, ErrorMessage.forbidden(\"Not authorized to update this profile\")}\n    end\n  end\n  \n  # Format user for API response\n  \n  defp format_profile(user) do\n    Map.take(user, [:id, :name, :email, :role])\n  end\nend\n```","ref":"error_handling_workflow.html#step-2-create-a-service-layer"},{"type":"extras","title":"Step 3: Create a Controller Layer - Building an Error Handling Workflow","doc":"Now, let's create a controller that would handle HTTP requests in a Phoenix application:\n\n```elixir\ndefmodule MyApp.UserController do\n  @moduledoc \"\"\"\n  Controller for user endpoints\n  \"\"\"\n  \n  # In a real Phoenix app, you would use: use MyApp.Web, :controller\n  \n  alias MyApp.UserService\n  \n  @doc \"\"\"\n  Get user profile\n  \"\"\"\n  def show(conn, %{\"id\" => id_string}) do\n    # Get the current user from the session/token (simplified for this example)\n    current_user = conn.assigns.current_user\n    \n    with {:ok, id} <- parse_id(id_string),\n         {:ok, profile} <- UserService.get_profile(id, current_user) do\n      # In a real Phoenix app: render(conn, \"show.json\", user: profile)\n      {:ok, profile}\n    else\n      {:error, error} -> render_error(conn, error)\n    end\n  end\n  \n  @doc \"\"\"\n  Update user profile\n  \"\"\"\n  def update(conn, %{\"id\" => id_string, \"user\" => user_params}) do\n    current_user = conn.assigns.current_user\n    \n    with {:ok, id} <- parse_id(id_string),\n         {:ok, profile} <- UserService.update_profile(id, user_params, current_user) do\n      # In a real Phoenix app: render(conn, \"show.json\", user: profile)\n      {:ok, profile}\n    else\n      {:error, error} -> render_error(conn, error)\n    end\n  end\n  \n  # Helper functions\n  \n  defp parse_id(id_string) do\n    case Integer.parse(id_string) do\n      {id, \"\"} -> {:ok, id}\n      _ -> {:error, ErrorMessage.bad_request(\"Invalid ID format\", %{id: id_string})}\n    end\n  end\n  \n  defp render_error(conn, %ErrorMessage{} = error) do\n    # In a real Phoenix app:\n    # conn\n    # |> put_status(ErrorMessage.http_code(error))\n    # |> put_view(MyApp.ErrorView)\n    # |> render(\"error.json\", error: error)\n    \n    # For this example, we'll just return the error\n    {:error, error}\n  end\nend\n```","ref":"error_handling_workflow.html#step-3-create-a-controller-layer"},{"type":"extras","title":"Step 4: Testing the Error Handling Flow - Building an Error Handling Workflow","doc":"Let's create a module to test our error handling flow:\n\n```elixir\ndefmodule MyApp.ErrorHandlingDemo do\n  @moduledoc \"\"\"\n  Demo module to show the error handling flow\n  \"\"\"\n  \n  alias MyApp.UserController\n  \n  @doc \"\"\"\n  Run the demo with different scenarios\n  \"\"\"\n  def run do\n    # Setup mock connection and users\n    admin_conn = %{assigns: %{current_user: %{id: 1, role: :admin}}}\n    user_conn = %{assigns: %{current_user: %{id: 2, role: :user}}}\n    \n    IO.puts(\"\\n=== Scenario 1: Admin accessing own profile ===\")\n    case UserController.show(admin_conn, %{\"id\" => \"1\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 2: Admin accessing another user's profile ===\")\n    case UserController.show(admin_conn, %{\"id\" => \"2\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 3: Regular user accessing own profile ===\")\n    case UserController.show(user_conn, %{\"id\" => \"2\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 4: Regular user accessing admin's profile ===\")\n    case UserController.show(user_conn, %{\"id\" => \"1\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 5: Invalid ID format ===\")\n    case UserController.show(admin_conn, %{\"id\" => \"abc\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 6: Non-existent user ===\")\n    case UserController.show(admin_conn, %{\"id\" => \"999\"}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n    \n    IO.puts(\"\\n=== Scenario 7: Update with invalid role ===\")\n    case UserController.update(admin_conn, %{\"id\" => \"2\", \"user\" => %{\"role\" => \"superuser\"}}) do\n      {:ok, profile} -> IO.puts(\"Success: #{inspect(profile)}\")\n      {:error, error} -> IO.puts(\"Error: #{error}\")\n    end\n  end\nend\n```","ref":"error_handling_workflow.html#step-4-testing-the-error-handling-flow"},{"type":"extras","title":"Step 5: Running the Demo - Building an Error Handling Workflow","doc":"To run the demo, you would create a script or run it in IEx:\n\n```elixir\n# In a script or IEx session\nMyApp.ErrorHandlingDemo.run()\n```\n\nThis would produce output showing how different error scenarios are handled:\n\n```\n=== Scenario 1: Admin accessing own profile ===\nSuccess: %{email: \"alice@example.com\", id: 1, name: \"Alice\", role: :admin}\n\n=== Scenario 2: Admin accessing another user's profile ===\nSuccess: %{email: \"bob@example.com\", id: 2, name: \"Bob\", role: :user}\n\n=== Scenario 3: Regular user accessing own profile ===\nSuccess: %{email: \"bob@example.com\", id: 2, name: \"Bob\", role: :user}\n\n=== Scenario 4: Regular user accessing admin's profile ===\nError: forbidden - Not authorized to view this profile\n\n=== Scenario 5: Invalid ID format ===\nError: bad_request - Invalid ID format\nDetails: \n%{id: \"abc\"}\n\n=== Scenario 6: Non-existent user ===\nError: not_found - User not found\nDetails: \n%{user_id: 999}\n\n=== Scenario 7: Update with invalid role ===\nError: unprocessable_entity - Invalid role\nDetails: \n%{allowed_roles: [:admin, :user], provided_role: \"superuser\"}\n```","ref":"error_handling_workflow.html#step-5-running-the-demo"},{"type":"extras","title":"Step 6: Logging Errors - Building an Error Handling Workflow","doc":"Let's enhance our error handling with proper logging:\n\n```elixir\ndefmodule MyApp.ErrorLogger do\n  require Logger\n  \n  @doc \"\"\"\n  Log an error with appropriate severity based on the error code\n  \"\"\"\n  def log_error(%ErrorMessage{} = error, context) do\n    log_level = get_log_level(error.code)\n    Logger.log(log_level, \"[#{context}] #{error}\")\n    error\n  end\n  \n  defp get_log_level(code) when code in [:internal_server_error, :service_unavailable] do\n    :error\n  end\n  \n  defp get_log_level(code) when code in [:bad_request, :unauthorized, :forbidden, :not_found] do\n    :info\n  end\n  \n  defp get_log_level(_code) do\n    :warn\n  end\nend\n```\n\nThen update the controller to use the logger:\n\n```elixir\ndefp render_error(conn, %ErrorMessage{} = error) do\n  # Log the error\n  MyApp.ErrorLogger.log_error(error, \"UserController\")\n  \n  # Render the error response\n  # ...\nend\n```","ref":"error_handling_workflow.html#step-6-logging-errors"},{"type":"extras","title":"Conclusion - Building an Error Handling Workflow","doc":"In this tutorial, we've built a complete error handling workflow using ErrorMessage:\n\n1. **Domain Layer**: Used ErrorMessage to represent domain-specific errors\n2. **Service Layer**: Added business logic and authorization with appropriate error messages\n3. **Controller Layer**: Handled user input validation and converted errors to HTTP responses\n4. **Logging**: Added contextual error logging with appropriate severity levels\n\nThis approach provides several benefits:\n\n- **Consistent Error Structure**: All errors follow the same format\n- **Rich Error Context**: Errors include detailed information to help with debugging\n- **Appropriate HTTP Status Codes**: Errors map directly to HTTP status codes\n- **Pattern Matching**: Easy to handle specific error types with pattern matching\n- **Logging Integration**: Structured errors are easy to log and analyze\n\nBy following this pattern, you can create robust, maintainable applications with clear error handling throughout all layers of your system.","ref":"error_handling_workflow.html#conclusion"},{"type":"extras","title":"Getting Started with ErrorMessage","doc":"# Getting Started with ErrorMessage\n\nThis tutorial will guide you through the basics of using ErrorMessage in your Elixir application. By the end, you'll understand how to create, handle, and serialize error messages.","ref":"getting_started.html"},{"type":"extras","title":"Prerequisites - Getting Started with ErrorMessage","doc":"- Basic knowledge of Elixir\n- An Elixir project set up with Mix","ref":"getting_started.html#prerequisites"},{"type":"extras","title":"Step 1: Add ErrorMessage to Your Project - Getting Started with ErrorMessage","doc":"First, add ErrorMessage to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:error_message, \"~> 0.3.2\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting_started.html#step-1-add-errormessage-to-your-project"},{"type":"extras","title":"Step 2: Creating Your First Error Message - Getting Started with ErrorMessage","doc":"Let's start by creating a simple error message:\n\n```elixir\n# Create a basic not_found error\nerror = ErrorMessage.not_found(\"Resource not found\")\n```\n\nThis creates an `%ErrorMessage{}` struct with the code `:not_found` and the message \"Resource not found\".","ref":"getting_started.html#step-2-creating-your-first-error-message"},{"type":"extras","title":"Step 3: Adding Details to Your Error - Getting Started with ErrorMessage","doc":"Error messages can include additional details to provide more context:\n\n```elixir\n# Create an error with details\nerror = ErrorMessage.not_found(\"User not found\", %{user_id: 123})\n```\n\nThe details can be any Elixir term, but keep in mind that if you plan to convert the error to JSON, you should use data structures that can be serialized.","ref":"getting_started.html#step-3-adding-details-to-your-error"},{"type":"extras","title":"Step 4: Using Error Messages in Function Returns - Getting Started with ErrorMessage","doc":"A common pattern is to return `{:ok, result}` or `{:error, error_message}` from functions:\n\n```elixir\ndefmodule UserRepository do\n  def find_user(id) do\n    case Database.get_user(id) do\n      nil -> {:error, ErrorMessage.not_found(\"User not found\", %{user_id: id})}\n      user -> {:ok, user}\n    end\n  end\nend\n```","ref":"getting_started.html#step-4-using-error-messages-in-function-returns"},{"type":"extras","title":"Step 5: Pattern Matching on Error Codes - Getting Started with ErrorMessage","doc":"One of the benefits of ErrorMessage is the ability to pattern match on error codes:\n\n```elixir\ncase UserRepository.find_user(123) do\n  {:ok, user} -> \n    # Process the user\n    IO.puts(\"Found user: #{user.name}\")\n    \n  {:error, %ErrorMessage{code: :not_found}} ->\n    # Handle not found specifically\n    IO.puts(\"User not found, creating a new one\")\n    create_new_user(123)\n    \n  {:error, error} ->\n    # Handle other errors\n    IO.puts(\"Error: #{error.message}\")\nend\n```","ref":"getting_started.html#step-5-pattern-matching-on-error-codes"},{"type":"extras","title":"Step 6: Converting Errors to Strings for Logging - Getting Started with ErrorMessage","doc":"ErrorMessage implements the `String.Chars` protocol, making it easy to convert errors to strings for logging:\n\n```elixir\nrequire Logger\n\ndef process_user(id) do\n  case UserRepository.find_user(id) do\n    {:ok, user} -> \n      {:ok, process(user)}\n      \n    {:error, error} = result ->\n      Logger.error(\"Failed to find user: #{error}\")\n      result\n  end\nend\n```","ref":"getting_started.html#step-6-converting-errors-to-strings-for-logging"},{"type":"extras","title":"Step 7: Converting Errors to JSON for API Responses - Getting Started with ErrorMessage","doc":"When building web APIs, you can easily convert error messages to JSON:\n\n```elixir\ndefmodule MyAPI.ErrorView do\n  def render(\"error.json\", %{error: error}) do\n    ErrorMessage.to_jsonable_map(error)\n  end\nend\n```","ref":"getting_started.html#step-7-converting-errors-to-json-for-api-responses"},{"type":"extras","title":"Next Steps - Getting Started with ErrorMessage","doc":"Now that you understand the basics of ErrorMessage, you can:\n\n- Check out the [How-To Guides](https://hexdocs.pm/error_message/how-to-guides.html) for practical examples\n- Explore the [Explanation](https://hexdocs.pm/error_message/explanation.html) section to understand the design principles\n- Refer to the [API Reference](https://hexdocs.pm/error_message/api-reference.html) for detailed function documentation\n\nCongratulations! You've completed the getting started tutorial for ErrorMessage.","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Common Error Handling Patterns","doc":"# Common Error Handling Patterns\n\nThis guide demonstrates common patterns for handling errors with the ErrorMessage library in different contexts.","ref":"error_handling_patterns.html"},{"type":"extras","title":"Pattern Matching on Error Codes - Common Error Handling Patterns","doc":"One of the most powerful features of ErrorMessage is the ability to pattern match on error codes:\n\n```elixir\ndef process_result(result) do\n  # Generally we won't be returning atoms through our\n  # system but as an example of how we can handle them\n\n  case result do\n    {:ok, value} ->\n      # Handle success case\n      {:ok, process_value(value)}\n      \n    {:error, %ErrorMessage{code: :not_found}} ->\n      # Handle not found specifically\n      {:error, :resource_missing}\n      \n    {:error, %ErrorMessage{code: :unauthorized}} ->\n      # Handle unauthorized specifically\n      {:error, :permission_denied}\n      \n    {:error, %ErrorMessage{} = error} ->\n      # Handle any other error\n      Logger.error(\"Unexpected error: #{error}\")\n      {:error, :unexpected_error}\n  end\nend\n```","ref":"error_handling_patterns.html#pattern-matching-on-error-codes"},{"type":"extras","title":"Using with/1 for Error Handling - Common Error Handling Patterns","doc":"The `with/1` special form in Elixir works great with ErrorMessage for handling multiple operations that might fail:\n\n```elixir\ndef create_order(user_id, product_id, quantity) do\n  with {:ok, user} <- find_user(user_id),\n       {:ok, product} <- find_product(product_id),\n       {:ok, _} <- check_inventory(product, quantity),\n       {:ok, order} <- create_order_record(user, product, quantity) do\n    {:ok, order}\n  else\n    {:error, %ErrorMessage{code: :not_found, details: %{user_id: _}}} ->\n      {:error, ErrorMessage.bad_request(\"Invalid user\", %{user_id: user_id})}\n      \n    {:error, %ErrorMessage{code: :not_found, details: %{product_id: _}}} ->\n      {:error, ErrorMessage.bad_request(\"Invalid product\", %{product_id: product_id})}\n      \n    {:error, error} ->\n      # Pass through any other errors\n      {:error, error}\n  end\nend\n```","ref":"error_handling_patterns.html#using-with-1-for-error-handling"},{"type":"extras","title":"Converting Other Error Types to ErrorMessage - Common Error Handling Patterns","doc":"Often you'll need to convert errors from other libraries to ErrorMessage format:\n\n```elixir\ndef handle_ecto_errors(changeset) do\n  errors = Ecto.Changeset.traverse_errors(changeset, fn {msg, opts} ->\n    Enum.reduce(opts, msg, fn {key, value}, acc ->\n      String.replace(acc, \"%{#{key}}\", to_string(value))\n    end)\n  end)\n  \n  ErrorMessage.unprocessable_entity(\"Validation failed\", errors)\nend\n\ndef handle_file_errors(file_path) do\n  case File.read(file_path) do\n    {:ok, content} ->\n      {:ok, content}\n      \n    {:error, :enoent} ->\n      {:error, ErrorMessage.not_found(\"File not found\", %{path: file_path})}\n      \n    {:error, :eacces} ->\n      {:error, ErrorMessage.forbidden(\"Permission denied\", %{path: file_path})}\n      \n    {:error, reason} ->\n      {:error, ErrorMessage.internal_server_error(\"File error\", %{reason: reason, path: file_path})}\n  end\nend\n```","ref":"error_handling_patterns.html#converting-other-error-types-to-errormessage"},{"type":"extras","title":"Error Handling in GenServers - Common Error Handling Patterns","doc":"ErrorMessage works well in GenServer implementations:\n\n```elixir\ndefmodule MyApp.UserManager do\n  use GenServer\n  \n  # Client API\n  \n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n  \n  def get_user(id) do\n    GenServer.call(__MODULE__, {:get_user, id})\n  end\n  \n  # Server Callbacks\n  \n  def init(opts) do\n    {:ok, %{users: opts[:initial_users] || %{}}}\n  end\n  \n  def handle_call({:get_user, id}, _from, state) do\n    case Map.get(state.users, id) do\n      nil ->\n        {:reply, {:error, ErrorMessage.not_found(\"User not found\", %{user_id: id})}, state}\n        \n      user ->\n        {:reply, {:ok, user}, state}\n    end\n  end\nend\n```","ref":"error_handling_patterns.html#error-handling-in-genservers"},{"type":"extras","title":"Composing Error Handlers - Common Error Handling Patterns","doc":"You can create helper functions to handle common error patterns:\n\n```elixir\ndefmodule MyApp.ErrorHelpers do\n  require Logger\n  \n  def with_logging(result, context) do\n    case result do\n      {:ok, _} = success ->\n        success\n        \n      {:error, %ErrorMessage{} = error} ->\n        Logger.error(\"[#{context}] #{error}\")\n        {:error, error}\n    end\n  end\n  \n  def with_fallback(result, fallback_fn) do\n    case result do\n      {:ok, _} = success ->\n        success\n        \n      {:error, %ErrorMessage{code: :not_found}} ->\n        fallback_fn.()\n        \n      {:error, _} = error ->\n        error\n    end\n  end\n  \n  def with_retry(operation, retry_count \\\\ 3) do\n    retry_with_count(operation, retry_count)\n  end\n  \n  defp retry_with_count(operation, count) when count > 0 do\n    case operation.() do\n      {:ok, _} = success ->\n        success\n        \n      {:error, %ErrorMessage{code: code} = error} when code in [:service_unavailable, :gateway_timeout] ->\n        # Only retry on certain errors\n        Process.sleep(100)\n        retry_with_count(operation, count - 1)\n        \n      {:error, _} = error ->\n        error\n    end\n  end\n  \n  defp retry_with_count(_operation, 0) do\n    {:error, ErrorMessage.service_unavailable(\"Operation failed after multiple retries\")}\n  end\nend\n```\n\nUsage:\n\n```elixir\nimport MyApp.ErrorHelpers\n\ndef process_user(id) do\n  with_logging(\n    with_fallback(\n      UserRepository.find_user(id),\n      fn -> UserRepository.create_default_user(id) end\n    ),\n    \"UserProcessing\"\n  )\nend\n\ndef fetch_remote_data(url) do\n  with_retry(fn -> ApiClient.get(url) end)\nend\n```","ref":"error_handling_patterns.html#composing-error-handlers"},{"type":"extras","title":"Handling Errors in Concurrent Operations - Common Error Handling Patterns","doc":"When working with concurrent operations, you can collect and process errors:\n\n```elixir\ndef process_items(items) do\n  items\n  |> Enum.map(fn item ->\n    Task.async(fn -> process_item(item) end)\n  end)\n  |> Task.await_many()\n  |> Enum.split_with(fn\n    {:ok, _} -> true\n    {:error, _} -> false\n  end)\n  |> case do\n    {successes, []} ->\n      # All operations succeeded\n      {:ok, Enum.map(successes, fn {:ok, result} -> result end)}\n      \n    {_, errors} ->\n      # Some operations failed\n      error_details = Enum.map(errors, fn {:error, error} -> \n        %{item_id: error.details.item_id, reason: error.message}\n      end)\n      \n      {:error, ErrorMessage.multi_status(\"Some operations failed\", %{errors: error_details})}\n  end\nend\n```\n\nThese patterns should help you effectively use ErrorMessage in various scenarios throughout your application.","ref":"error_handling_patterns.html#handling-errors-in-concurrent-operations"},{"type":"extras","title":"How to Integrate ErrorMessage with Phoenix","doc":"# How to Integrate ErrorMessage with Phoenix\n\nThis guide demonstrates how to effectively use ErrorMessage in a Phoenix application to handle errors consistently across your API.","ref":"phoenix_integration.html"},{"type":"extras","title":"Setting Up Error Handling in Controllers - How to Integrate ErrorMessage with Phoenix","doc":"Create a helper module or function to handle error responses consistently:\n\n```elixir\ndefmodule MyApp.ErrorHandler do\n  import Plug.Conn\n  import Phoenix.Controller\n\n  @doc \"\"\"\n  Renders an error response using the ErrorMessage struct\n  \"\"\"\n  def render_error(conn, %ErrorMessage{} = error) do\n    conn\n    |> put_status(ErrorMessage.http_code(error))\n    |> put_view(MyApp.ErrorView)\n    |> render(\"error.json\", error: error)\n  end\nend\n```","ref":"phoenix_integration.html#setting-up-error-handling-in-controllers"},{"type":"extras","title":"Creating an Error View - How to Integrate ErrorMessage with Phoenix","doc":"In your Phoenix application, create or update your error view to handle ErrorMessage structs:\n\n```elixir\ndefmodule MyApp.ErrorView do\n  use MyApp, :view\n\n  @doc \"\"\"\n  Renders an error message from an ErrorMessage struct\n  \"\"\"\n  def render(\"error.json\", %{error: error}) do\n    ErrorMessage.to_jsonable_map(error)\n  end\n\n  # Keep your existing error templates\n  def render(\"404.json\", _assigns) do\n    %{errors: %{detail: \"Not Found\"}}\n  end\n\n  def render(\"500.json\", _assigns) do\n    %{errors: %{detail: \"Internal Server Error\"}}\n  end\nend\n```","ref":"phoenix_integration.html#creating-an-error-view"},{"type":"extras","title":"Using in Controllers - How to Integrate ErrorMessage with Phoenix","doc":"Now you can use ErrorMessage in your controllers:\n\n```elixir\ndefmodule MyApp.UserController do\n  use MyApp, :controller\n  import MyApp.ErrorHandler\n\n  def show(conn, %{\"id\" => id}) do\n    case MyApp.Accounts.get_user(id) do\n      {:ok, user} ->\n        render(conn, \"show.json\", user: user)\n        \n      {:error, %ErrorMessage{} = error} ->\n        render_error(conn, error)\n    end\n  end\n  \n  def create(conn, %{\"user\" => user_params}) do\n    case MyApp.Accounts.create_user(user_params) do\n      {:ok, user} ->\n        conn\n        |> put_status(:created)\n        |> render(\"show.json\", user: user)\n        \n      {:error, %ErrorMessage{} = error} ->\n        render_error(conn, error)\n    end\n  end\nend\n```","ref":"phoenix_integration.html#using-in-controllers"},{"type":"extras","title":"Creating a Fallback Controller - How to Integrate ErrorMessage with Phoenix","doc":"For more advanced error handling, you can create a fallback controller:\n\n```elixir\ndefmodule MyApp.FallbackController do\n  use Phoenix.Controller\n  import MyApp.ErrorHandler\n\n  def call(conn, {:error, %ErrorMessage{} = error}) do\n    render_error(conn, error)\n  end\n\n  # Handle other error types\n  def call(conn, {:error, %Ecto.Changeset{} = changeset}) do\n    # Convert Ecto.Changeset errors to ErrorMessage\n    errors = Ecto.Changeset.traverse_errors(changeset, &translate_error/1)\n    error = ErrorMessage.unprocessable_entity(\"Invalid parameters\", errors)\n    \n    render_error(conn, error)\n  end\n  \n  defp translate_error({msg, opts}) do\n    Enum.reduce(opts, msg, fn {key, value}, acc ->\n      String.replace(acc, \"%{#{key}}\", to_string(value))\n    end)\n  end\nend\n```\n\nThen in your controller:\n\n```elixir\ndefmodule MyApp.UserController do\n  use MyApp, :controller\n  \n  action_fallback MyApp.FallbackController\n  \n  def show(conn, %{\"id\" => id}) do\n    with {:ok, user} <- MyApp.Accounts.get_user(id) do\n      render(conn, \"show.json\", user: user)\n    end\n  end\nend\n```","ref":"phoenix_integration.html#creating-a-fallback-controller"},{"type":"extras","title":"Adding Request ID to Error Responses - How to Integrate ErrorMessage with Phoenix","doc":"ErrorMessage automatically includes the request ID in the error response if it's available in the Logger metadata. To ensure this works:\n\n1. Make sure you have the RequestId plug enabled in your endpoint:\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_app\n  \n  plug Plug.RequestId\n  # ...\nend\n```\n\n2. Ensure Logger is configured to include the request_id in metadata:\n\n```elixir\nconfig :logger, :console,\n  format: \"$time $metadata[$level] $message\\n\",\n  metadata: [:request_id]\n```\n\nWith this setup, all your API error responses will automatically include the request ID, making it easier to correlate errors with specific requests in your logs.","ref":"phoenix_integration.html#adding-request-id-to-error-responses"},{"type":"extras","title":"API Reference","doc":"# API Reference\n\nThis reference document provides detailed information about the ErrorMessage API, including all available functions, their parameters, and return types.","ref":"api_reference.html"},{"type":"extras","title":"Core Module: `ErrorMessage` - API Reference","doc":"The main module that provides functions for creating and manipulating error messages.","ref":"api_reference.html#core-module-errormessage"},{"type":"extras","title":"Error Creation Functions - API Reference","doc":"ErrorMessage provides a function for each HTTP status code. Here are some of the most commonly used ones:\n\n#### Client Error Functions\n\n```elixir\nErrorMessage.bad_request(message, details \\\\ nil)\nErrorMessage.unauthorized(message, details \\\\ nil)\nErrorMessage.forbidden(message, details \\\\ nil)\nErrorMessage.not_found(message, details \\\\ nil)\nErrorMessage.method_not_allowed(message, details \\\\ nil)\nErrorMessage.not_acceptable(message, details \\\\ nil)\nErrorMessage.request_timeout(message, details \\\\ nil)\nErrorMessage.conflict(message, details \\\\ nil)\nErrorMessage.gone(message, details \\\\ nil)\nErrorMessage.unprocessable_entity(message, details \\\\ nil)\nErrorMessage.too_many_requests(message, details \\\\ nil)\n```\n\n#### Server Error Functions\n\n```elixir\nErrorMessage.internal_server_error(message, details \\\\ nil)\nErrorMessage.not_implemented(message, details \\\\ nil)\nErrorMessage.bad_gateway(message, details \\\\ nil)\nErrorMessage.service_unavailable(message, details \\\\ nil)\nErrorMessage.gateway_timeout(message, details \\\\ nil)\n```\n\nEach function creates an `%ErrorMessage{}` struct with the appropriate error code, message, and optional details.","ref":"api_reference.html#error-creation-functions"},{"type":"extras","title":"Utility Functions - API Reference","doc":"#### `to_string/1`\n\nConverts an error message to a string representation.\n\n```elixir\n@spec to_string(error_message :: t) :: String.t()\n```\n\nExample:\n```elixir\niex> ErrorMessage.to_string(ErrorMessage.not_found(\"User not found\"))\n\"not_found - User not found\"\n\niex> ErrorMessage.to_string(ErrorMessage.internal_server_error(\"Error\", %{reason: :timeout}))\n\"internal_server_error - Error\\nDetails: \\n%{reason: :timeout}\"\n```\n\n#### `to_jsonable_map/1`\n\nConverts an error message to a map suitable for JSON serialization.\n\n```elixir\n@spec to_jsonable_map(error_message :: t) :: t_map\n```\n\nExample:\n```elixir\niex> ErrorMessage.to_jsonable_map(ErrorMessage.not_found(\"User not found\", %{id: 123}))\n%{code: :not_found, message: \"User not found\", details: %{id: 123}}\n```\n\nIf a request ID is available in the Logger metadata, it will be included in the map:\n\n```elixir\n%{\n  code: :not_found,\n  message: \"User not found\",\n  details: %{id: 123},\n  request_id: \"FzMx0iBDvDDJ-GkAAAfh\"\n}\n```\n\n#### `http_code/1`\n\nReturns the HTTP status code for an error message or error code atom.\n\n```elixir\n@spec http_code(error_code :: code) :: non_neg_integer()\n@spec http_code(error_message :: t) :: non_neg_integer()\n```\n\nExample:\n```elixir\niex> ErrorMessage.http_code(:not_found)\n404\n\niex> ErrorMessage.http_code(ErrorMessage.internal_server_error(\"Error\"))\n500\n```\n\n#### `http_code_reason_atom/1`\n\nReturns the HTTP reason as an atom for the HTTP error code.\n\n```elixir\n@spec http_code_reason_atom(error_code :: non_neg_integer()) :: code\n```\n\nExample:\n```elixir\niex> ErrorMessage.http_code_reason_atom(404)\n:not_found\n\niex> ErrorMessage.http_code_reason_atom(500)\n:internal_server_error\n```","ref":"api_reference.html#utility-functions"},{"type":"extras","title":"Types - API Reference","doc":"### `ErrorMessage.t`\n\nThe main error message struct type.\n\n```elixir\n@type t :: %ErrorMessage{code: code, message: String.t(), details: any()}\n@type t(details) :: %ErrorMessage{code: code, message: String.t(), details: details}\n```\n\n### `ErrorMessage.code`\n\nThe error code type, which is an atom representing an HTTP status code.\n\n```elixir\n@type code :: :multiple_choices\n            | :moved_permanently\n            | :found\n            | :see_other\n            | :not_modified\n            | :use_proxy\n            | :switch_proxy\n            | :temporary_redirect\n            | :permanent_redirect\n            | :bad_request\n            | :unauthorized\n            | :payment_required\n            | :forbidden\n            | :not_found\n            | :method_not_allowed\n            | :not_acceptable\n            | :proxy_authentication_required\n            | :request_timeout\n            | :conflict\n            | :gone\n            | :length_required\n            | :precondition_failed\n            | :request_entity_too_large\n            | :request_uri_too_long\n            | :unsupported_media_type\n            | :requested_range_not_satisfiable\n            | :expectation_failed\n            | :im_a_teapot\n            | :misdirected_request\n            | :unprocessable_entity\n            | :locked\n            | :failed_dependency\n            | :too_early\n            | :upgrade_required\n            | :precondition_required\n            | :too_many_requests\n            | :request_header_fields_too_large\n            | :unavailable_for_legal_reasons\n            | :internal_server_error\n            | :not_implemented\n            | :bad_gateway\n            | :service_unavailable\n            | :gateway_timeout\n            | :http_version_not_supported\n            | :variant_also_negotiates\n            | :insufficient_storage\n            | :loop_detected\n            | :not_extended\n            | :network_authentication_required\n```","ref":"api_reference.html#types"},{"type":"extras","title":"Result Types - API Reference","doc":"ErrorMessage provides several type specifications for common result patterns:\n\n```elixir\n@type t_res :: {:ok, term} | {:error, t}\n@type t_res(result_type) :: {:ok, result_type} | {:error, t}\n@type t_res(result_type, details_type) :: {:ok, result_type} | {:error, t(details_type)}\n\n@type t_ok_res :: :ok | {:error, t}\n@type t_ok_res(details_type) :: :ok | {:error, t(details_type)}\n```","ref":"api_reference.html#result-types"},{"type":"extras","title":"Map Types - API Reference","doc":"Types for the map representation of error messages:\n\n```elixir\n@type t_map :: %{code: code, message: String.t(), details: any(), request_id: String.t()} |\n               %{code: code, message: String.t(), details: any()}\n\n@type t_map(details) :: %{code: code, message: String.t(), details: details, request_id: String.t()} |\n                        %{code: code, message: String.t(), details: details}\n```","ref":"api_reference.html#map-types"},{"type":"extras","title":"Protocols - API Reference","doc":"### `String.Chars`\n\nErrorMessage implements the `String.Chars` protocol, allowing error messages to be converted to strings using `to_string/1` or string interpolation:\n\n```elixir\ndefimpl String.Chars do\n  def to_string(%ErrorMessage{} = e) do\n    ErrorMessage.to_string(e)\n  end\nend\n```\n\nExample:\n```elixir\niex> \"Error: #{ErrorMessage.not_found(\"User not found\")}\"\n\"Error: not_found - User not found\"\n```\n\n### `Jason.Encoder` (Optional)\n\nIf the Jason library is available, ErrorMessage automatically implements the `Jason.Encoder` protocol, allowing error messages to be directly encoded to JSON:\n\n```elixir\nif Enum.any?(Application.loaded_applications(), fn {dep_name, _, _} -> dep_name === :jason end) do\n  @derive Jason.Encoder\nend\n```\n\nExample:\n```elixir\niex> Jason.encode!(ErrorMessage.not_found(\"User not found\"))\n\"{\\\"code\\\":\\\"not_found\\\",\\\"message\\\":\\\"User not found\\\"}\"\n```","ref":"api_reference.html#protocols"},{"type":"extras","title":"Error Serialization Reference","doc":"# Error Serialization Reference\n\nThis document provides detailed information about how ErrorMessage handles serialization of error messages to different formats.","ref":"serialization.html"},{"type":"extras","title":"String Serialization - Error Serialization Reference","doc":"ErrorMessage implements the `String.Chars` protocol, which allows error messages to be converted to strings using `to_string/1` or string interpolation.","ref":"serialization.html#string-serialization"},{"type":"extras","title":"Basic Format - Error Serialization Reference","doc":"The string representation of an error message follows this format:\n\n- For errors without details: `\"#{code} - #{message}\"`\n- For errors with details: `\"#{code} - #{message}\\nDetails: \\n#{inspect(details, pretty: true)}\"`","ref":"serialization.html#basic-format"},{"type":"extras","title":"Examples - Error Serialization Reference","doc":"```elixir\n# Error without details\nerror = ErrorMessage.not_found(\"User not found\")\nto_string(error)  # \"not_found - User not found\"\n\n# Error with details\nerror = ErrorMessage.internal_server_error(\"Database error\", %{table: \"users\", reason: :connection_lost})\nto_string(error)\n# \"internal_server_error - Database error\n# Details: \n# %{reason: :connection_lost, table: \"users\"}\"\n\n# String interpolation\n\"Error occurred: #{error}\"\n# \"Error occurred: internal_server_error - Database error\n# Details: \n# %{reason: :connection_lost, table: \"users\"}\"\n```","ref":"serialization.html#examples"},{"type":"extras","title":"JSON Serialization - Error Serialization Reference","doc":"ErrorMessage provides the `to_jsonable_map/1` function to convert error messages to maps suitable for JSON serialization.","ref":"serialization.html#json-serialization"},{"type":"extras","title":"Basic Format - Error Serialization Reference","doc":"The JSON representation of an error message follows this format:\n\n```json\n{\n  \"code\": \"error_code\",\n  \"message\": \"Error message\",\n  \"details\": { ... }\n}\n```\n\nIf a request ID is available in the Logger metadata, it will be included:\n\n```json\n{\n  \"code\": \"error_code\",\n  \"message\": \"Error message\",\n  \"details\": { ... },\n  \"request_id\": \"FzMx0iBDvDDJ-GkAAAfh\"\n}\n```","ref":"serialization.html#basic-format"},{"type":"extras","title":"Data Type Handling - Error Serialization Reference","doc":"The ErrorMessage.Serializer module includes the `ensure_json_serializable/1` function, which handles conversion of Elixir-specific data types to JSON-compatible formats:\n\n| Elixir Type | JSON Representation |\n|-------------|---------------------|\n| `Date` | ISO 8601 date string |\n| `Time` | ISO 8601 time string |\n| `DateTime` | ISO 8601 datetime string |\n| `NaiveDateTime` | ISO 8601 datetime string |\n| `Struct` | `{\"struct\": \"StructName\", \"data\": {...}}` |\n| `Tuple` | Array |\n| `PID` | String representation (with registered name if available) |\n| `Function` | `{\"module\": \"Module\", \"function\": \"function_name\", \"arity\": n}` |\n| `List` | Array with each element converted recursively |\n| `Map` | Object with each value converted recursively |\n| Other types | Passed through as-is |","ref":"serialization.html#data-type-handling"},{"type":"extras","title":"Examples - Error Serialization Reference","doc":"```elixir\n# Basic error\nerror = ErrorMessage.not_found(\"User not found\", %{user_id: 123})\nErrorMessage.to_jsonable_map(error)\n# %{code: :not_found, message: \"User not found\", details: %{user_id: 123}}\n\n# Error with complex details\nerror = ErrorMessage.bad_request(\"Invalid data\", %{\n  date: ~D[2023-01-15],\n  time: ~T[14:30:00],\n  callback: &String.length/1,\n  user: %UserStruct{name: \"John\", created_at: ~N[2023-01-01 00:00:00]}\n})\n\nErrorMessage.to_jsonable_map(error)\n# %{\n#   code: :bad_request,\n#   message: \"Invalid data\",\n#   details: %{\n#     date: \"2023-01-15\",\n#     time: \"14:30:00\",\n#     callback: %{module: \"Elixir.String\", function: \"length\", arity: 1},\n#     user: %{\n#       struct: \"UserStruct\",\n#       data: %{\n#         name: \"John\",\n#         created_at: \"2023-01-01T00:00:00\"\n#       }\n#     }\n#   }\n# }\n```","ref":"serialization.html#examples"},{"type":"extras","title":"Direct JSON Encoding - Error Serialization Reference","doc":"If the Jason library is available, ErrorMessage automatically implements the `Jason.Encoder` protocol, allowing error messages to be directly encoded to JSON:\n\n```elixir\n# With Jason available\nJason.encode!(ErrorMessage.not_found(\"User not found\"))\n# \"{\\\"code\\\":\\\"not_found\\\",\\\"message\\\":\\\"User not found\\\"}\"\n\n# With details\nJason.encode!(ErrorMessage.not_found(\"User not found\", %{user_id: 123}))\n# \"{\\\"code\\\":\\\"not_found\\\",\\\"message\\\":\\\"User not found\\\",\\\"details\\\":{\\\"user_id\\\":123}}\"\n```\n\nNote that when using `Jason.encode!/1` directly on an `%ErrorMessage{}` struct, the serialization of complex data types in the details field may not be handled as thoroughly as when using `ErrorMessage.to_jsonable_map/1` first. For the most reliable JSON serialization, especially with complex data structures, use:\n\n```elixir\nerror\n|> ErrorMessage.to_jsonable_map()\n|> Jason.encode!()\n```","ref":"serialization.html#direct-json-encoding"},{"type":"extras","title":"Integration with Phoenix - Error Serialization Reference","doc":"When using ErrorMessage with Phoenix, you can easily render error messages as JSON responses:\n\n```elixir\ndefmodule MyApp.ErrorView do\n  use MyApp, :view\n  \n  def render(\"error.json\", %{error: error}) do\n    ErrorMessage.to_jsonable_map(error)\n  end\nend\n```\n\nThis will produce JSON responses with the appropriate structure and data type handling.","ref":"serialization.html#integration-with-phoenix"},{"type":"extras","title":"Error Design Principles","doc":"# Error Design Principles\n\nThis document explains the design philosophy and principles behind the ErrorMessage library, helping you understand why it was created and how it can improve your application's error handling.","ref":"error_design_principles.html"},{"type":"extras","title":"Why Consistent Error Handling Matters - Error Design Principles","doc":"Error handling is a critical aspect of any robust application. Inconsistent error handling can lead to:\n\n1. **Unpredictable behavior**: When errors are represented differently across your application, it becomes difficult to predict how errors will be structured.\n2. **Fragile error handling**: String-based error matching is brittle and can break when error messages change.\n3. **Poor user experience**: Inconsistent error reporting leads to confusing user interfaces and API responses.\n4. **Difficult debugging**: Without structured errors, it's harder to track down the root cause of issues.\n\nErrorMessage addresses these problems by providing a consistent structure for all errors in your application.","ref":"error_design_principles.html#why-consistent-error-handling-matters"},{"type":"extras","title":"HTTP Status Codes as a Universal Language - Error Design Principles","doc":"ErrorMessage uses HTTP status codes as the foundation for error classification. This approach has several advantages:\n\n1. **Universality**: HTTP status codes are widely understood across programming languages and platforms.\n2. **Semantic meaning**: Each status code has a well-defined meaning (e.g., 404 for \"not found\", 403 for \"forbidden\").\n3. **Standardization**: Using HTTP status codes means your error system aligns with web standards.\n4. **API compatibility**: When building web APIs, your internal error representation maps directly to HTTP responses.\n\nEven for non-web applications, HTTP status codes provide a comprehensive and well-understood taxonomy of error conditions that can be applied to many domains.","ref":"error_design_principles.html#http-status-codes-as-a-universal-language"},{"type":"extras","title":"The Three-Part Error Structure - Error Design Principles","doc":"ErrorMessage uses a three-part structure for errors:\n\n1. **Code**: An atom representing the error type (e.g., `:not_found`, `:internal_server_error`)\n2. **Message**: A human-readable description of what went wrong\n3. **Details**: Additional context-specific information about the error\n\nThis structure provides a balance between:\n\n- **Simplicity**: The core structure is easy to understand and use\n- **Flexibility**: The details field can contain any data structure needed for your specific use case\n- **Consistency**: All errors follow the same pattern, making them predictable","ref":"error_design_principles.html#the-three-part-error-structure"},{"type":"extras","title":"Pattern Matching vs. String Matching - Error Design Principles","doc":"One of the key benefits of ErrorMessage is the ability to pattern match on error codes rather than error messages:\n\n```elixir\n# Fragile approach (depends on exact message text)\ncase result do\n  {:error, \"User not found: \" <> _} -> handle_not_found()\n  # ...\nend\n\n# Robust approach with ErrorMessage\ncase result do\n  {:error, %ErrorMessage{code: :not_found}} -> handle_not_found()\n  # ...\nend\n```\n\nPattern matching on error codes is:\n\n1. **More robust**: Changing error messages doesn't break your error handling logic\n2. **More explicit**: The intent of your code is clearer\n3. **More maintainable**: Error handling code is less likely to break over time","ref":"error_design_principles.html#pattern-matching-vs-string-matching"},{"type":"extras","title":"Serialization Considerations - Error Design Principles","doc":"ErrorMessage includes built-in support for converting errors to strings (for logging) and maps (for JSON serialization). This addresses common challenges with error serialization:\n\n1. **Complex data structures**: The `ensure_json_serializable/1` function handles conversion of Elixir-specific data types (like PIDs, Date/Time structs, etc.) to JSON-compatible formats.\n2. **Request context**: When available, the request ID is automatically included in serialized errors, making it easier to correlate errors with specific requests.\n3. **Consistent output**: All errors are serialized in a consistent format, making it easier to process them in clients.","ref":"error_design_principles.html#serialization-considerations"},{"type":"extras","title":"Integration with Existing Systems - Error Design Principles","doc":"ErrorMessage is designed to integrate seamlessly with:\n\n- **Phoenix**: For web API error responses\n- **Logger**: For structured error logging\n- **Plug**: For HTTP status code mapping\n\nThis integration-friendly design means you can adopt ErrorMessage incrementally in your application without having to rewrite existing code all at once.","ref":"error_design_principles.html#integration-with-existing-systems"},{"type":"extras","title":"When to Use ErrorMessage - Error Design Principles","doc":"ErrorMessage is particularly valuable in:\n\n1. **API-driven applications**: Where consistent error responses are crucial\n2. **Microservice architectures**: Where errors might cross service boundaries\n3. **Large codebases**: Where consistency across modules is important\n4. **Long-lived applications**: Where maintainability over time is a priority\n\nWhile ErrorMessage can be used in any Elixir application, these scenarios highlight where it provides the most value.","ref":"error_design_principles.html#when-to-use-errormessage"},{"type":"extras","title":"Error Handling in Elixir and ErrorMessage","doc":"# Error Handling in Elixir and ErrorMessage\n\nThis document explains how error handling typically works in Elixir and how ErrorMessage enhances the standard approach.","ref":"error_handling_in_elixir.html"},{"type":"extras","title":"Traditional Error Handling in Elixir - Error Handling in Elixir and ErrorMessage","doc":"Elixir, like Erlang, follows the \"Let it crash\" philosophy, which encourages developers to focus on the happy path and let supervisors handle failures. However, this doesn't mean that we should ignore errors entirely. For expected errors that are part of normal program flow, Elixir provides several conventions:","ref":"error_handling_in_elixir.html#traditional-error-handling-in-elixir"},{"type":"extras","title":"Return Values - Error Handling in Elixir and ErrorMessage","doc":"The most common pattern in Elixir for handling errors is to return tagged tuples:\n\n```elixir\n# Success case\n{:ok, result}\n\n# Error case\n{:error, reason}\n```\n\nThis pattern allows for easy pattern matching:\n\n```elixir\ncase MyModule.some_function() do\n  {:ok, result} -> handle_success(result)\n  {:error, reason} -> handle_error(reason)\nend\n```","ref":"error_handling_in_elixir.html#return-values"},{"type":"extras","title":"Exceptions - Error Handling in Elixir and ErrorMessage","doc":"Elixir also supports exceptions for exceptional circumstances:\n\n```elixir\ntry do\n  some_function_that_might_raise()\nrescue\n  e in SomeError -> handle_error(e)\nend\n```\n\nHowever, exceptions are generally used for unexpected errors rather than as a control flow mechanism.","ref":"error_handling_in_elixir.html#exceptions"},{"type":"extras","title":"Limitations of Traditional Approaches - Error Handling in Elixir and ErrorMessage","doc":"While these approaches work well for simple cases, they have limitations:\n\n1. **Inconsistent Error Structure**: The `:error` tuple can contain any term as its reason, leading to inconsistent error handling across different parts of an application.\n\n2. **String-Based Error Messages**: Many libraries use simple strings as error reasons, which makes pattern matching fragile.\n\n3. **Limited Context**: Error reasons often lack detailed context about what went wrong.\n\n4. **No Standard for HTTP Integration**: When building web applications, there's no standard way to map errors to HTTP status codes.","ref":"error_handling_in_elixir.html#limitations-of-traditional-approaches"},{"type":"extras","title":"How ErrorMessage Improves Error Handling - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage addresses these limitations by providing:","ref":"error_handling_in_elixir.html#how-errormessage-improves-error-handling"},{"type":"extras","title":"1. Consistent Structure - Error Handling in Elixir and ErrorMessage","doc":"All errors follow the same structure:\n\n```elixir\n%ErrorMessage{\n  code: :error_code,\n  message: \"Human-readable message\",\n  details: additional_context\n}\n```\n\nThis consistency makes error handling more predictable across your application.","ref":"error_handling_in_elixir.html#1-consistent-structure"},{"type":"extras","title":"2. Code-Based Pattern Matching - Error Handling in Elixir and ErrorMessage","doc":"Instead of matching on error messages (which might change), you can match on error codes:\n\n```elixir\ncase result do\n  {:ok, value} -> handle_success(value)\n  {:error, %ErrorMessage{code: :not_found}} -> handle_not_found()\n  {:error, %ErrorMessage{code: :unauthorized}} -> handle_unauthorized()\n  {:error, _} -> handle_other_errors()\nend\n```","ref":"error_handling_in_elixir.html#2-code-based-pattern-matching"},{"type":"extras","title":"3. Rich Context - Error Handling in Elixir and ErrorMessage","doc":"The `details` field can contain any data structure, allowing you to provide rich context about what went wrong:\n\n```elixir\nErrorMessage.not_found(\"User not found\", %{\n  user_id: id,\n  search_params: params,\n  timestamp: DateTime.utc_now()\n})\n```","ref":"error_handling_in_elixir.html#3-rich-context"},{"type":"extras","title":"4. HTTP Integration - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage is built around HTTP status codes, making it easy to integrate with web applications:\n\n```elixir\ndef render_error(conn, %ErrorMessage{} = error) do\n  conn\n  |> put_status(ErrorMessage.http_code(error))\n  |> json(ErrorMessage.to_jsonable_map(error))\nend\n```","ref":"error_handling_in_elixir.html#4-http-integration"},{"type":"extras","title":"ErrorMessage in the Elixir Ecosystem - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage complements other Elixir error handling approaches:","ref":"error_handling_in_elixir.html#errormessage-in-the-elixir-ecosystem"},{"type":"extras","title":"With Standard Libraries - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage works alongside standard Elixir patterns:\n\n```elixir\ndef find_user(id) do\n  case Repo.get(User, id) do\n    nil -> {:error, ErrorMessage.not_found(\"User not found\", %{user_id: id})}\n    user -> {:ok, user}\n  end\nend\n```","ref":"error_handling_in_elixir.html#with-standard-libraries"},{"type":"extras","title":"With Ecto - Error Handling in Elixir and ErrorMessage","doc":"You can convert Ecto changeset errors to ErrorMessage:\n\n```elixir\ndef create_user(params) do\n  %User{}\n  |> User.changeset(params)\n  |> Repo.insert()\n  |> case do\n    {:ok, user} -> \n      {:ok, user}\n    {:error, changeset} ->\n      errors = Ecto.Changeset.traverse_errors(changeset, &translate_error/1)\n      {:error, ErrorMessage.unprocessable_entity(\"Invalid user data\", errors)}\n  end\nend\n```","ref":"error_handling_in_elixir.html#with-ecto"},{"type":"extras","title":"With Phoenix - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage integrates well with Phoenix for API error handling:\n\n```elixir\ndefmodule MyApp.FallbackController do\n  use Phoenix.Controller\n  \n  def call(conn, {:error, %ErrorMessage{} = error}) do\n    conn\n    |> put_status(ErrorMessage.http_code(error))\n    |> put_view(MyApp.ErrorView)\n    |> render(\"error.json\", error: error)\n  end\nend\n```","ref":"error_handling_in_elixir.html#with-phoenix"},{"type":"extras","title":"When to Use ErrorMessage - Error Handling in Elixir and ErrorMessage","doc":"ErrorMessage is particularly valuable in:\n\n1. **API-driven applications**: Where consistent error responses are crucial\n2. **Complex business logic**: Where detailed error context helps with debugging\n3. **Cross-module boundaries**: Where consistent error handling improves maintainability\n4. **Web applications**: Where mapping errors to HTTP status codes is important\n\nMost systems will benefit from ErrorMessage providing significant benefits for applications that need structured, consistent error handling, as well as benefitting pattern matching on error codes.\n\nFor further reading checkout this [blog post](https://learn-elixir.dev/blogs/safer-error-systems-in-elixir)","ref":"error_handling_in_elixir.html#when-to-use-errormessage"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}